@@ -33,12 +33,12 @@ typedef enum
 {
   EPHY_DOWNLOAD_ACTION_NONE,
   EPHY_DOWNLOAD_ACTION_BROWSE_TO,
-  EPHY_DOWNLOAD_ACTION_OPEN,
-  EPHY_DOWNLOAD_ACTION_DO_NOT_AUTO_OPEN
+  EPHY_DOWNLOAD_ACTION_OPEN
 } EphyDownloadActionType;
 
 EphyDownload *ephy_download_new                   (WebKitDownload *download);
 EphyDownload *ephy_download_new_for_uri           (const char     *uri);
+EphyDownload *ephy_download_new_for_uri_internal  (const char     *uri);
 
 void          ephy_download_cancel                (EphyDownload *download);
 gboolean      ephy_download_is_active             (EphyDownload *download);
