@@ -33,13 +33,31 @@ typedef struct {
   const char *schema;
   const char *key;
   const char *webkit_pref;
-  void (*callback)(GSettings *settings, const char *key, gpointer data);
+  void (*callback)(GSettings  *settings,
+                   const char *key,
+                   gpointer    data);
 } PrefData;
 
 #define DEFAULT_ENCODING_SETTING "default-charset"
-// FIXME: Refactor this code to remove the need of those globals
+/* FIXME: Refactor this code to remove the need of those globals */
 static WebKitSettings *webkit_settings = NULL;
 static GFileMonitor *user_style_sheet_monitor = NULL;
+static WebKitUserStyleSheet *style_sheet = NULL;
+static GList *ucm_list = NULL;
+
+static void
+update_user_style_on_all_ucm (void)
+{
+  GList *list = NULL;
+
+  for (list = ucm_list; list != NULL; list = list->next) {
+    WebKitUserContentManager *ucm = list->data;
+
+    webkit_user_content_manager_remove_all_style_sheets (ucm);
+    if (style_sheet)
+      webkit_user_content_manager_add_style_sheet (ucm, style_sheet);
+  }
+}
 
 static void
 user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
@@ -48,17 +66,18 @@ user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
 {
   gssize bytes;
 
+  g_clear_pointer (&style_sheet, webkit_user_style_sheet_unref);
+
   bytes = g_output_stream_splice_finish (output_stream, result, NULL);
   if (bytes > 0) {
-    WebKitUserStyleSheet *style_sheet;
-
     style_sheet = webkit_user_style_sheet_new (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)),
                                                WEBKIT_USER_CONTENT_INJECT_ALL_FRAMES, WEBKIT_USER_STYLE_LEVEL_USER,
                                                NULL, NULL);
-    webkit_user_content_manager_add_style_sheet (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())),
-                                                 style_sheet);
-    webkit_user_style_sheet_unref (style_sheet);
   }
+
+  update_user_style_on_all_ucm ();
+
+  g_object_unref (output_stream);
 }
 
 static void
@@ -66,23 +85,21 @@ user_style_sheet_read_cb (GFile        *file,
                           GAsyncResult *result,
                           gpointer      user_data)
 {
-  GFileInputStream *input_stream;
-  GOutputStream *output_stream;
+  g_autoptr (GFileInputStream) input_stream = NULL;
+  g_autoptr (GOutputStream) output_stream = NULL;
 
   input_stream = g_file_read_finish (file, result, NULL);
   if (!input_stream)
     return;
 
   output_stream = g_memory_output_stream_new_resizable ();
-  g_output_stream_splice_async (output_stream, G_INPUT_STREAM (input_stream),
-                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
-                                G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+  g_output_stream_splice_async (g_steal_pointer (&output_stream),
+                                G_INPUT_STREAM (input_stream),
+                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
                                 G_PRIORITY_DEFAULT,
                                 NULL,
                                 (GAsyncReadyCallback)user_style_sheet_output_stream_splice_cb,
                                 NULL);
-  g_object_unref (input_stream);
-  g_object_unref (output_stream);
 }
 
 static void
@@ -93,8 +110,6 @@ user_style_sheet_file_changed (GFileMonitor      *monitor,
                                gpointer           user_data)
 {
   if (event_type == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT) {
-    webkit_user_content_manager_remove_all_style_sheets (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())));
-
     g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
                        (GAsyncReadyCallback)user_style_sheet_read_cb, NULL);
   }
@@ -105,35 +120,33 @@ webkit_pref_callback_user_stylesheet (GSettings  *settings,
                                       const char *key,
                                       gpointer    data)
 {
+  g_autoptr (GFile) file = NULL;
+  g_autofree char *filename = NULL;
   gboolean value;
+  GError *error = NULL;
 
   value = g_settings_get_boolean (settings, key);
 
+  g_clear_object (&user_style_sheet_monitor);
+  g_clear_pointer (&style_sheet, webkit_user_style_sheet_unref);
+
   if (!value) {
-    g_clear_object (&user_style_sheet_monitor);
-    webkit_user_content_manager_remove_all_style_sheets (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())));
+    update_user_style_on_all_ucm ();
+    return;
+  }
+
+  filename = g_build_filename (ephy_profile_dir (), USER_STYLESHEET_FILENAME, NULL);
+  file = g_file_new_for_path (filename);
+
+  g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
+                     (GAsyncReadyCallback)user_style_sheet_read_cb, NULL);
+
+  user_style_sheet_monitor = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &error);
+  if (!user_style_sheet_monitor) {
+    g_warning ("Could not create a file monitor for %s: %s\n", g_file_get_uri (file), error->message);
+    g_error_free (error);
   } else {
-    GFile *file;
-    GError *error = NULL;
-    char *filename;
-
-    filename = g_build_filename (ephy_profile_dir (), USER_STYLESHEET_FILENAME, NULL);
-    file = g_file_new_for_path (filename);
-    g_free (filename);
-
-    g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
-                       (GAsyncReadyCallback)user_style_sheet_read_cb, NULL);
-
-    g_assert (user_style_sheet_monitor == NULL);
-    user_style_sheet_monitor = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &error);
-    if (user_style_sheet_monitor == NULL) {
-      g_warning ("Could not create a file monitor for %s: %s\n", g_file_get_uri (file), error->message);
-      g_error_free (error);
-    } else {
-      g_signal_connect (user_style_sheet_monitor, "changed", G_CALLBACK (user_style_sheet_file_changed), NULL);
-    }
-
-    g_object_unref (file);
+    g_signal_connect (user_style_sheet_monitor, "changed", G_CALLBACK (user_style_sheet_file_changed), NULL);
   }
 }
 
@@ -142,17 +155,8 @@ webkit_pref_callback_user_agent (GSettings  *settings,
                                  const char *key,
                                  gpointer    data)
 {
-  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
-  char *user_agent;
-
-  if (ephy_embed_shell_get_mode (shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
-    user_agent = g_strdup_printf ("%s (Web App)", ephy_user_agent_get_internal ());
-  else
-    user_agent = g_strdup (ephy_user_agent_get_internal ());
-
-  webkit_settings_set_user_agent (webkit_settings, user_agent);
-
-  g_free (user_agent);
+  webkit_settings_set_user_agent (webkit_settings,
+                                  ephy_user_agent_get ());
 }
 
 static void
@@ -254,7 +258,7 @@ normalize_languages (char **languages)
 }
 
 /* Based on Christian Persch's code from gecko backend of epiphany
-   (old transform_accept_languages_list() function) */
+ *  (old transform_accept_languages_list() function) */
 static void
 webkit_pref_callback_accept_languages (GSettings  *settings,
                                        const char *key,
@@ -413,7 +417,7 @@ webkit_pref_callback_hardware_acceleration_policy (GSettings  *settings,
                                                         WEBKIT_HARDWARE_ACCELERATION_POLICY_NEVER);
       break;
     case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_ON_DEMAND:
-      /* Fallthrough */
+    /* Fallthrough */
     default:
       webkit_settings_set_hardware_acceleration_policy (webkit_settings,
                                                         WEBKIT_HARDWARE_ACCELERATION_POLICY_ON_DEMAND);
@@ -421,8 +425,7 @@ webkit_pref_callback_hardware_acceleration_policy (GSettings  *settings,
   }
 }
 
-static const PrefData webkit_pref_entries[] =
-{
+static const PrefData webkit_pref_entries[] = {
   /* Epiphany font settings */
   { EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_SERIF_FONT,
@@ -471,6 +474,11 @@ static const PrefData webkit_pref_entries[] =
     EPHY_PREFS_WEB_USER_AGENT,
     "user-agent",
     webkit_pref_callback_user_agent },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_MOBILE_USER_AGENT,
+    "mobile-user-agent",
+    /* No need for a specific callback, we just want to update the user agent. */
+    webkit_pref_callback_user_agent },
   { EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_COOKIES_POLICY,
     "accept-policy",
@@ -486,13 +494,16 @@ ephy_embed_prefs_init (gpointer user_data)
 {
   guint i;
 
-  webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
-                                                       "enable-fullscreen", TRUE,
-                                                       "enable-javascript", TRUE,
+  webkit_settings = webkit_settings_new_with_settings ("enable-back-forward-navigation-gestures", TRUE,
+                                                       "enable-developer-extras", TRUE,
                                                        "enable-dns-prefetching", TRUE,
-                                                       "enable-mediasource", TRUE,
+                                                       "enable-plugins", FALSE,
+                                                       "enable-webaudio", TRUE, /* FIXME: https://bugs.webkit.org/show_bug.cgi?id=205334 */
+                                                       "enable-webgl", TRUE, /* FIXME: https://bugs.webkit.org/show_bug.cgi?id=205335 */
                                                        "javascript-can-open-windows-automatically", TRUE,
-                                                       "enable-back-forward-navigation-gestures", TRUE,
+#if WEBKIT_CHECK_VERSION (2, 27, 90) && !WEBKIT_CHECK_VERSION (2, 30, 0)
+                                                       "allow-top-navigation-to-data-urls", FALSE,
+#endif
                                                        NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
@@ -516,10 +527,6 @@ ephy_embed_prefs_init (gpointer user_data)
                    EPHY_PREFS_ENABLE_CARET_BROWSING,
                    webkit_settings, "enable-caret-browsing",
                    G_SETTINGS_BIND_GET);
-  g_settings_bind (EPHY_SETTINGS_WEB,
-                   EPHY_PREFS_WEB_ENABLE_PLUGINS,
-                   webkit_settings, "enable-plugins",
-                   G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_FONT_MIN_SIZE,
                    webkit_settings, "minimum-font-size",
@@ -528,14 +535,6 @@ ephy_embed_prefs_init (gpointer user_data)
                    EPHY_PREFS_WEB_DEFAULT_ENCODING,
                    webkit_settings, DEFAULT_ENCODING_SETTING,
                    G_SETTINGS_BIND_GET);
-  g_settings_bind (EPHY_SETTINGS_WEB,
-                   EPHY_PREFS_WEB_ENABLE_WEBGL,
-                   webkit_settings, "enable-webgl",
-                   G_SETTINGS_BIND_GET);
-  g_settings_bind (EPHY_SETTINGS_WEB,
-                   EPHY_PREFS_WEB_ENABLE_WEBAUDIO,
-                   webkit_settings, "enable-webaudio",
-                   G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING,
                    webkit_settings, "enable-smooth-scrolling",
@@ -554,3 +553,22 @@ ephy_embed_prefs_get_settings (void)
 
   return g_once (&once_init, ephy_embed_prefs_init, NULL);
 }
+
+void
+ephy_embed_prefs_apply_user_style (WebKitUserContentManager *ucm)
+{
+  if (style_sheet)
+    webkit_user_content_manager_add_style_sheet (ucm, style_sheet);
+}
+
+void
+ephy_embed_prefs_register_ucm (WebKitUserContentManager *ucm)
+{
+  ucm_list = g_list_append (ucm_list, ucm);
+}
+
+void
+ephy_embed_prefs_unregister_ucm (WebKitUserContentManager *ucm)
+{
+  ucm_list = g_list_remove (ucm_list, ucm);
+}
