@@ -46,8 +46,6 @@ char *   ephy_embed_utils_normalize_or_autosearch_address       (const char *add
 gboolean ephy_embed_utils_url_is_empty                          (const char *location);
 gboolean ephy_embed_utils_is_no_show_address                    (const char *address);
 char    *ephy_embed_utils_get_title_from_address                (const char *address);
-gboolean ephy_embed_utils_urls_have_same_origin                 (const char *a_url,
-                                                                 const char *b_url);
 void     ephy_embed_utils_shutdown                              (void);
 
 G_END_DECLS
