@@ -132,7 +132,8 @@ static GParamSpec *obj_properties[LAST_PROP];
  */
 
 static guint
-ephy_embed_statusbar_get_context_id (EphyEmbed *embed, const char  *context_description)
+ephy_embed_statusbar_get_context_id (EphyEmbed  *embed,
+                                     const char *context_description)
 {
   char *string;
   guint id;
@@ -155,7 +156,8 @@ ephy_embed_statusbar_get_context_id (EphyEmbed *embed, const char  *context_desc
 }
 
 static void
-ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label)
+ephy_embed_set_statusbar_label (EphyEmbed  *embed,
+                                const char *label)
 {
   nautilus_floating_bar_set_primary_label (NAUTILUS_FLOATING_BAR (embed->floating_bar), label);
 
@@ -167,7 +169,8 @@ ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label)
 }
 
 static void
-ephy_embed_statusbar_update (EphyEmbed *embed, const char *text)
+ephy_embed_statusbar_update (EphyEmbed  *embed,
+                             const char *text)
 {
   g_assert (EPHY_IS_EMBED (embed));
 
@@ -175,7 +178,9 @@ ephy_embed_statusbar_update (EphyEmbed *embed, const char *text)
 }
 
 static guint
-ephy_embed_statusbar_push (EphyEmbed *embed, guint context_id, const char *text)
+ephy_embed_statusbar_push (EphyEmbed  *embed,
+                           guint       context_id,
+                           const char *text)
 {
   EphyEmbedStatusbarMsg *msg;
 
@@ -198,7 +203,8 @@ ephy_embed_statusbar_push (EphyEmbed *embed, guint context_id, const char *text)
 /* End of code based on GTK+ GtkStatusbar. */
 
 static void
-ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
+ephy_embed_statusbar_pop (EphyEmbed *embed,
+                          guint      context_id)
 {
   EphyEmbedStatusbarMsg *msg;
   GSList *list;
@@ -223,7 +229,8 @@ ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
 }
 
 static void
-remove_from_destroy_list_cb (GtkWidget *widget, EphyEmbed *embed)
+remove_from_destroy_list_cb (GtkWidget *widget,
+                             EphyEmbed *embed)
 {
   GSList *list;
 
@@ -283,9 +290,9 @@ web_view_title_changed_cb (WebKitWebView *web_view,
 }
 
 static void
-load_changed_cb (WebKitWebView  *web_view,
-                 WebKitLoadEvent load_event,
-                 EphyEmbed      *embed)
+load_changed_cb (WebKitWebView   *web_view,
+                 WebKitLoadEvent  load_event,
+                 EphyEmbed       *embed)
 {
   switch (load_event) {
     case WEBKIT_LOAD_COMMITTED:
@@ -334,9 +341,7 @@ ephy_embed_entering_fullscreen (EphyEmbed *embed)
   if (!g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN, EPHY_PREFS_LOCKDOWN_FULLSCREEN)) {
     gtk_widget_show (embed->fullscreen_message_label);
 
-    if (embed->fullscreen_message_id)
-      g_source_remove (embed->fullscreen_message_id);
-
+    g_clear_handle_id (&embed->fullscreen_message_id, g_source_remove);
     embed->fullscreen_message_id = g_timeout_add_seconds (5,
                                                           (GSourceFunc)fullscreen_message_label_hide,
                                                           embed);
@@ -355,37 +360,13 @@ ephy_embed_dispose (GObject *object)
 {
   EphyEmbed *embed = EPHY_EMBED (object);
 
-  if (embed->pop_statusbar_later_source_id) {
-    g_source_remove (embed->pop_statusbar_later_source_id);
-    embed->pop_statusbar_later_source_id = 0;
-  }
+  g_clear_handle_id (&embed->pop_statusbar_later_source_id, g_source_remove);
+  g_clear_handle_id (&embed->clear_progress_source_id, g_source_remove);
+  g_clear_handle_id (&embed->delayed_request_source_id, g_source_remove);
+  g_clear_handle_id (&embed->fullscreen_message_id, g_source_remove);
 
-  if (embed->clear_progress_source_id) {
-    g_source_remove (embed->clear_progress_source_id);
-    embed->clear_progress_source_id = 0;
-  }
-
-  if (embed->delayed_request_source_id) {
-    g_source_remove (embed->delayed_request_source_id);
-    embed->delayed_request_source_id = 0;
-  }
-
-  /* Do not listen to status message notifications anymore, if we try
-   * to update the statusbar after dispose we might crash. */
-  if (embed->status_handler_id) {
-    g_signal_handler_disconnect (embed->web_view, embed->status_handler_id);
-    embed->status_handler_id = 0;
-  }
-
-  if (embed->progress_update_handler_id) {
-    g_signal_handler_disconnect (embed->web_view, embed->progress_update_handler_id);
-    embed->progress_update_handler_id = 0;
-  }
-
-  if (embed->fullscreen_message_id) {
-    g_source_remove (embed->fullscreen_message_id);
-    embed->fullscreen_message_id = 0;
-  }
+  g_clear_signal_handler (&embed->status_handler_id, embed->web_view);
+  g_clear_signal_handler (&embed->progress_update_handler_id, embed->web_view);
 
   g_clear_object (&embed->delayed_request);
   g_clear_pointer (&embed->delayed_state, webkit_web_view_session_state_unref);
@@ -536,7 +517,7 @@ ephy_embed_attach_inspector_cb (WebKitWebInspector *inspector,
 
 static gboolean
 ephy_embed_close_inspector_cb (WebKitWebInspector *inspector,
-                                EphyEmbed          *embed)
+                               EphyEmbed          *embed)
 {
   embed->inspector_loaded = FALSE;
 
@@ -583,23 +564,21 @@ pop_statusbar_later_cb (gpointer data)
 }
 
 static void
-status_message_notify_cb (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed)
+status_message_notify_cb (EphyWebView *view,
+                          GParamSpec  *pspec,
+                          EphyEmbed   *embed)
 {
   const char *message;
 
   message = ephy_web_view_get_status_message (view);
 
   if (message) {
-    if (embed->pop_statusbar_later_source_id) {
-      g_source_remove (embed->pop_statusbar_later_source_id);
-      embed->pop_statusbar_later_source_id = 0;
-    }
-
+    g_clear_handle_id (&embed->pop_statusbar_later_source_id, g_source_remove);
     ephy_embed_statusbar_pop (embed, embed->tab_message_id);
     ephy_embed_statusbar_push (embed, embed->tab_message_id, message);
   } else {
     /* A short timeout before hiding the statusbar ensures that while moving
-       over a series of links, the overlay widget doesn't flicker on and off. */
+     *  over a series of links, the overlay widget doesn't flicker on and off. */
     if (embed->pop_statusbar_later_source_id == 0) {
       embed->pop_statusbar_later_source_id = g_timeout_add (250, pop_statusbar_later_cb, embed);
       g_source_set_name_by_id (embed->pop_statusbar_later_source_id, "[epiphany] pop_statusbar_later_cb");
@@ -617,16 +596,15 @@ clear_progress_cb (EphyEmbed *embed)
 }
 
 static void
-progress_update (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed)
+progress_update (EphyWebView *view,
+                 GParamSpec  *pspec,
+                 EphyEmbed   *embed)
 {
   gdouble progress;
   gboolean loading;
   const char *uri;
 
-  if (embed->clear_progress_source_id) {
-    g_source_remove (embed->clear_progress_source_id);
-    embed->clear_progress_source_id = 0;
-  }
+  g_clear_handle_id (&embed->clear_progress_source_id, g_source_remove);
 
   uri = webkit_web_view_get_uri (embed->web_view);
   if (!uri || g_str_has_prefix (uri, "ephy-about:") ||
@@ -675,11 +653,6 @@ load_delayed_request_if_mapped (gpointer user_data)
   g_clear_object (&embed->delayed_request);
   g_clear_pointer (&embed->delayed_state, webkit_web_view_session_state_unref);
 
-  /* This is to allow UI elements watching load status to show that the page is
-   * loading as soon as possible.
-   */
-  g_signal_emit_by_name (web_view, "load-changed", WEBKIT_LOAD_STARTED);
-
   return G_SOURCE_REMOVE;
 }
 
@@ -698,7 +671,8 @@ ephy_embed_maybe_load_delayed_request (EphyEmbed *embed)
 }
 
 static void
-ephy_embed_restored_window_cb (EphyEmbedShell *shell, EphyEmbed *embed)
+ephy_embed_restored_window_cb (EphyEmbedShell *shell,
+                               EphyEmbed      *embed)
 {
   if (!gtk_widget_get_mapped (GTK_WIDGET (embed)))
     return;
@@ -707,7 +681,8 @@ ephy_embed_restored_window_cb (EphyEmbedShell *shell, EphyEmbed *embed)
 }
 
 static void
-ephy_embed_mapped_cb (GtkWidget *widget, gpointer data)
+ephy_embed_mapped_cb (GtkWidget *widget,
+                      gpointer   data)
 {
   ephy_embed_maybe_load_delayed_request ((EphyEmbed *)widget);
 }
@@ -911,7 +886,8 @@ ephy_embed_add_top_widget (EphyEmbed                *embed,
  * details.
  */
 void
-ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
+ephy_embed_remove_top_widget (EphyEmbed *embed,
+                              GtkWidget *widget)
 {
   if (g_slist_find (embed->destroy_on_transition_list, widget)) {
     GSList *list;
@@ -936,7 +912,9 @@ ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
  * is on is switched to.
  */
 void
-ephy_embed_set_delayed_load_request (EphyEmbed *embed, WebKitURIRequest *request, WebKitWebViewSessionState *state)
+ephy_embed_set_delayed_load_request (EphyEmbed                 *embed,
+                                     WebKitURIRequest          *request,
+                                     WebKitWebViewSessionState *state)
 {
   g_assert (EPHY_IS_EMBED (embed));
   g_assert (WEBKIT_IS_URI_REQUEST (request));
@@ -1018,3 +996,137 @@ ephy_embed_detach_notification_container (EphyEmbed *embed)
     gtk_container_remove (GTK_CONTAINER (embed->overlay), g_object_ref (GTK_WIDGET (container)));
   }
 }
+
+typedef struct {
+  char *document_uri;
+  char *remote_uri;
+  WebKitDownload *download;
+  EphyWebView *web_view;
+} PdfAsyncData;
+
+static PdfAsyncData *
+pdf_async_data_new (const char  *document_uri,
+                    const char  *remote_uri,
+                    EphyWebView *web_view)
+{
+  PdfAsyncData *data;
+
+  data = g_new0 (PdfAsyncData, 1);
+  data->document_uri = g_strdup (document_uri);
+  data->remote_uri = g_strdup (remote_uri);
+  data->web_view = web_view;
+
+  if (web_view)
+    g_object_add_weak_pointer (G_OBJECT (web_view), (gpointer *)&data->web_view);
+
+  return data;
+}
+
+static void
+pdf_async_data_free (PdfAsyncData *data)
+{
+  if (data->web_view)
+    g_object_remove_weak_pointer (G_OBJECT (data->web_view), (gpointer *)&data->web_view);
+
+  g_free (data->document_uri);
+  g_free (data->remote_uri);
+  g_free (data);
+}
+
+static void
+pdf_file_deleted (GObject      *source,
+                  GAsyncResult *res,
+                  gpointer      user_data)
+{
+  g_autoptr (GError) error = NULL;
+  if (!g_file_delete_finish (G_FILE (source), res, &error))
+    g_warning ("Could not delete temporary PDF file: %s", error->message);
+}
+
+static void
+pdf_file_loaded (GObject      *source,
+                 GAsyncResult *res,
+                 gpointer      user_data)
+{
+  PdfAsyncData *data = user_data;
+  WebKitWebView *web_view = WEBKIT_WEB_VIEW (data->web_view);
+  GBytes *html_file;
+  g_autoptr (GError) error = NULL;
+  g_autoptr (GString) html = NULL;
+  g_autofree gchar *b64 = NULL;
+  g_autofree gchar *requested_uri = NULL;
+  g_autofree char *file_data = NULL;
+  gsize len = 0;
+
+  if (!g_file_load_contents_finish (G_FILE (source), res, &file_data, &len, NULL, &error)) {
+    g_warning ("Could not read PDF file content: %s", error->message);
+    pdf_async_data_free (data);
+    return;
+  }
+
+  html_file = g_resources_lookup_data ("/org/gnome/epiphany/pdfjs/web/viewer.html", 0, NULL);
+
+  b64 = g_base64_encode ((const guchar *)file_data, len);
+  g_file_delete_async (G_FILE (source), G_PRIORITY_DEFAULT, NULL, pdf_file_deleted, NULL);
+
+  html = g_string_new ("");
+  g_string_printf (html, g_bytes_get_data (html_file, NULL), b64, g_path_get_basename (data->remote_uri));
+
+  webkit_web_view_load_alternate_html (web_view, html->str, data->remote_uri, "ephy-resource:///org/gnome/epiphany/pdfjs/web/");
+
+  /* FIXME: Necessary WebKit API to access security level (TLS certificate) of a download is not
+   * available. Please fix this line once available.
+   */
+  ephy_web_view_set_security_level (data->web_view, EPHY_SECURITY_LEVEL_LOCAL_PAGE);
+
+  pdf_async_data_free (data);
+}
+
+static void
+pdf_download_finished_cb (WebKitDownload *download,
+                          EphyEmbed      *embed)
+{
+  EphyWebView *view = ephy_embed_get_web_view (embed);
+  WebKitURIRequest *request = webkit_download_get_request (download);
+  g_autoptr (GFile) file = NULL;
+  PdfAsyncData *pdf_data;
+  const char *document_uri = webkit_download_get_destination (download);
+  const char *remote_uri = webkit_uri_request_get_uri (request);
+
+  file = g_file_new_for_uri (document_uri);
+
+  pdf_data = pdf_async_data_new (document_uri, remote_uri, view);
+  g_file_load_contents_async (file, NULL, pdf_file_loaded, pdf_data);
+}
+
+static gboolean
+pdf_download_decide_destination_cb (WebKitDownload *wk_download,
+                                    gchar          *suggested_filename,
+                                    gpointer        user_data)
+{
+  EphyWebView *web_view = ephy_embed_get_web_view (EPHY_EMBED (user_data));
+
+  if (ephy_web_view_in_pdf_viewer (web_view)) {
+    g_autofree gchar *tmp_file = g_strdup_printf ("%s/%s", g_get_tmp_dir (), suggested_filename);
+    g_autofree gchar *file_uri = g_filename_to_uri (tmp_file, NULL, NULL);
+
+    webkit_download_set_allow_overwrite (wk_download, TRUE);
+    webkit_download_set_destination (wk_download, file_uri);
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+void
+ephy_embed_download_started (EphyEmbed    *embed,
+                             EphyDownload *ephy_download)
+{
+  EphyWebView *web_view = ephy_embed_get_web_view (embed);
+  WebKitDownload *download = ephy_download_get_webkit_download (ephy_download);
+
+  if (ephy_web_view_in_pdf_viewer (web_view)) {
+    g_signal_connect_object (download, "finished", G_CALLBACK (pdf_download_finished_cb), embed, 0);
+    g_signal_connect_object (download, "decide-destination", G_CALLBACK (pdf_download_decide_destination_cb), embed, 0);
+  }
+}
