@@ -97,8 +97,8 @@ finish_uri_scheme_request (EphyViewSourceRequest *request,
   }
 
   request->source_handler->outstanding_requests =
-      g_list_remove (request->source_handler->outstanding_requests,
-                     request);
+    g_list_remove (request->source_handler->outstanding_requests,
+                   request);
 
   ephy_view_source_request_free (request);
 }
@@ -108,37 +108,34 @@ web_resource_data_cb (WebKitWebResource     *resource,
                       GAsyncResult          *result,
                       EphyViewSourceRequest *request)
 {
-  guchar *data;
-  char *data_str;
-  char *escaped_str;
-  char *html;
+  g_autofree guchar *data = NULL;
+  g_autofree char *escaped_str = NULL;
+  g_autoptr (GError) error = NULL;
+  g_autofree char *html = NULL;
   gsize length;
-  GError *error = NULL;
 
   data = webkit_web_resource_get_data_finish (resource, result, &length, &error);
-  if (error) {
+  if (!data) {
     finish_uri_scheme_request (request, NULL, error);
-    g_error_free (error);
     return;
   }
 
-  data_str = g_malloc (length + 1);
-  strncpy (data_str, (const char *)data, length);
-  data_str[length] = '\0';
-  g_free (data);
+  /* Warning: data is not a string, so we pass length here because it's not NUL-terminated. */
+  escaped_str = g_markup_escape_text ((const char *)data, length);
 
-  escaped_str = g_markup_escape_text (data_str, -1);
-  g_free (data_str);
-
-  html = g_strdup_printf ("<body>"
-                            "<pre>"
-                              "<code class=\"language-html\">%s</code>"
-                            "</pre>"
+  html = g_strdup_printf ("<head>"
+                          "  <link rel='stylesheet' href='ephy-resource:///org/gnome/epiphany/highlight.css'>"
+                          "  <title>%s</title>"
+                          "</head>"
+                          "<body class='hljs'>"
+                          "  <script src='ephy-resource:///org/gnome/epiphany/highlight.js'></script>"
+                          "  <script>hljs.initHighlightingOnLoad();</script>"
+                          "  <pre><code class='html'>%s</code></pre>"
                           "</body>",
+                          webkit_web_resource_get_uri (resource),
                           escaped_str);
-  g_free (escaped_str);
 
-  finish_uri_scheme_request (request, html, NULL);
+  finish_uri_scheme_request (request, g_steal_pointer (&html), NULL);
 }
 
 static void
@@ -158,8 +155,12 @@ load_changed_cb (WebKitWebView         *web_view,
                  WebKitLoadEvent        load_event,
                  EphyViewSourceRequest *request)
 {
-  if (load_event == WEBKIT_LOAD_FINISHED)
+  if (load_event == WEBKIT_LOAD_FINISHED) {
+    g_signal_handler_disconnect (request->web_view, request->load_changed_id);
+    request->load_changed_id = 0;
+
     ephy_view_source_request_begin_get_source_from_web_view (request, web_view);
+  }
 }
 
 static void
@@ -236,6 +237,7 @@ out:
 static void
 ephy_view_source_request_start (EphyViewSourceRequest *request)
 {
+  guint port;
   SoupURI *soup_uri;
   char *modified_uri;
   char *decoded_fragment;
@@ -243,7 +245,7 @@ ephy_view_source_request_start (EphyViewSourceRequest *request)
   WebKitWebView *web_view;
 
   request->source_handler->outstanding_requests =
-      g_list_prepend (request->source_handler->outstanding_requests, request);
+    g_list_prepend (request->source_handler->outstanding_requests, request);
 
   original_uri = webkit_uri_scheme_request_get_uri (request->scheme_request);
   soup_uri = soup_uri_new (original_uri);
@@ -259,9 +261,13 @@ ephy_view_source_request_start (EphyViewSourceRequest *request)
     return;
   }
 
-  /* Convert e.g. ephy-source://gnome.org#https to https://gnome.org */
+  /* Convert e.g. ephy-source://gnome.org#https to https://gnome.org, taking
+   * care to prevent soup_uri_set_scheme() from forcing the default port.
+   */
   decoded_fragment = soup_uri_decode (soup_uri->fragment);
+  port = soup_uri_get_port (soup_uri);
   soup_uri_set_scheme (soup_uri, decoded_fragment);
+  soup_uri_set_port (soup_uri, port);
   soup_uri_set_fragment (soup_uri, NULL);
   modified_uri = soup_uri_to_string (soup_uri, FALSE);
   g_assert (modified_uri);
