@@ -28,7 +28,8 @@ GtkWindowGroup *ephy_gui_ensure_window_group             (GtkWindow *window);
 
 void            ephy_gui_get_current_event               (GdkEventType *type,
                                                           guint *state,
-                                                          guint *button);
+                                                          guint *button,
+                                                          guint *keyval);
 
 void            ephy_gui_help                            (GtkWidget *parent,
                                                           const char *page);
