@@ -21,7 +21,6 @@
 #include "config.h"
 #include "ephy-snapshot-service.h"
 
-#include "ephy-favicon-helpers.h"
 #include "ephy-file-helpers.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -197,7 +196,7 @@ save_thumbnail (GdkPixbuf  *pixbuf,
   chmod (tmp_path, 0600);
   rename (tmp_path, path);
 
- out:
+out:
   if (error != NULL) {
     g_warning ("Failed to create thumbnail %s: %s", tmp_path, error->message);
     g_error_free (error);
@@ -213,13 +212,10 @@ save_thumbnail (GdkPixbuf  *pixbuf,
 }
 
 static GdkPixbuf *
-ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
-                                        cairo_surface_t *favicon)
+ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface)
 {
   GdkPixbuf *snapshot, *scaled;
   int orig_width, orig_height;
-  float orig_aspect_ratio, dest_aspect_ratio;
-  int x_offset, new_width = 0, new_height;
 
   orig_width = cairo_image_surface_get_width (surface);
   orig_height = cairo_image_surface_get_height (surface);
@@ -234,22 +230,7 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
                                       EPHY_THUMBNAIL_HEIGHT,
                                       GDK_INTERP_TILES);
   } else {
-    orig_aspect_ratio = orig_width / (float)orig_height;
-    dest_aspect_ratio = EPHY_THUMBNAIL_WIDTH / (float)EPHY_THUMBNAIL_HEIGHT;
-
-    if (orig_aspect_ratio > dest_aspect_ratio) {
-      /* Wider than taller, crop the sides. */
-      new_width = orig_height * dest_aspect_ratio;
-      new_height = orig_height;
-      x_offset = (orig_width - new_width) / 2;
-    } else {
-      /* Crop the bottom otherwise. */
-      new_width = orig_width;
-      new_height = orig_width / (float)dest_aspect_ratio;
-      x_offset = 0;
-    }
-
-    snapshot = gdk_pixbuf_get_from_surface (surface, x_offset, 0, new_width, new_height);
+    snapshot = gdk_pixbuf_get_from_surface (surface, 0, 0, orig_width, orig_height);
     scaled = gdk_pixbuf_scale_simple (snapshot,
                                       EPHY_THUMBNAIL_WIDTH,
                                       EPHY_THUMBNAIL_HEIGHT,
@@ -258,20 +239,6 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
 
   g_object_unref (snapshot);
 
-  x_offset = 6;
-  if (favicon) {
-    GdkPixbuf *fav_pixbuf;
-    int favicon_size = 16;
-    int y_offset = gdk_pixbuf_get_height (scaled) - favicon_size - x_offset;
-
-    fav_pixbuf = ephy_pixbuf_get_from_surface_scaled (favicon, favicon_size, favicon_size);
-    gdk_pixbuf_composite (fav_pixbuf, scaled,
-                          x_offset, y_offset, favicon_size, favicon_size,
-                          x_offset, y_offset, 1, 1,
-                          GDK_INTERP_NEAREST, 255);
-    g_object_unref (fav_pixbuf);
-  }
-
   return scaled;
 }
 
@@ -343,10 +310,10 @@ idle_cache_snapshot_path (gpointer user_data)
 }
 
 static void
-cache_snapshot_data_in_idle (EphySnapshotService  *service,
-                             const char           *url,
-                             const char           *path,
-                             EphySnapshotFreshness freshness)
+cache_snapshot_data_in_idle (EphySnapshotService   *service,
+                             const char            *url,
+                             const char            *path,
+                             EphySnapshotFreshness  freshness)
 {
   CacheData *data;
   data = g_new (CacheData, 1);
@@ -397,9 +364,9 @@ ephy_snapshot_service_save_snapshot_async (EphySnapshotService *service,
 }
 
 static char *
-ephy_snapshot_service_save_snapshot_finish (EphySnapshotService *service,
-                                            GAsyncResult        *result,
-                                            GError             **error)
+ephy_snapshot_service_save_snapshot_finish (EphySnapshotService  *service,
+                                            GAsyncResult         *result,
+                                            GError              **error)
 {
   g_assert (g_task_is_valid (result, service));
 
@@ -425,8 +392,7 @@ save_snapshot (cairo_surface_t *surface,
 {
   SnapshotAsyncData *data = g_task_get_task_data (task);
 
-  data->snapshot = ephy_snapshot_service_prepare_snapshot (surface,
-                                                           webkit_web_view_get_favicon (data->web_view));
+  data->snapshot = ephy_snapshot_service_prepare_snapshot (surface);
 
   ephy_snapshot_service_save_snapshot_async (g_task_get_source_object (task),
                                              data->snapshot,
@@ -473,7 +439,8 @@ retrieve_snapshot_from_web_view (GTask *task)
   webkit_web_view_get_snapshot (data->web_view,
                                 WEBKIT_SNAPSHOT_REGION_VISIBLE,
                                 WEBKIT_SNAPSHOT_OPTIONS_NONE,
-                                NULL, (GAsyncReadyCallback)on_snapshot_ready,
+                                g_task_get_cancellable (task),
+                                (GAsyncReadyCallback)on_snapshot_ready,
                                 task);
   return FALSE;
 }
@@ -490,29 +457,29 @@ webview_destroyed_cb (GtkWidget *web_view,
 }
 
 static void
-webview_load_changed_cb (WebKitWebView  *web_view,
-                         WebKitLoadEvent load_event,
-                         GTask          *task)
+webview_load_changed_cb (WebKitWebView   *web_view,
+                         WebKitLoadEvent  load_event,
+                         GTask           *task)
 {
   if (load_event != WEBKIT_LOAD_FINISHED)
     return;
 
   /* Load finished doesn't ensure that we actually have visible content yet,
-     so hold a bit before retrieving the snapshot. */
+   *  so hold a bit before retrieving the snapshot. */
   g_idle_add ((GSourceFunc)retrieve_snapshot_from_web_view, task);
 
   /* Some pages might end up causing this condition to happen twice, so remove
-     the handler in order to avoid calling the above idle function twice. */
+   *  the handler in order to avoid calling the above idle function twice. */
   g_signal_handlers_disconnect_by_func (web_view, webview_load_changed_cb, task);
   g_signal_handlers_disconnect_by_func (web_view, webview_destroyed_cb, task);
 }
 
 static gboolean
-webview_load_failed_cb (WebKitWebView  *web_view,
-                        WebKitLoadEvent load_event,
-                        const char      failing_uri,
-                        GError         *error,
-                        GTask          *task)
+webview_load_failed_cb (WebKitWebView   *web_view,
+                        WebKitLoadEvent  load_event,
+                        const char       failing_uri,
+                        GError          *error,
+                        GTask           *task)
 {
   g_signal_handlers_disconnect_by_func (web_view, webview_load_changed_cb, task);
   g_signal_handlers_disconnect_by_func (web_view, webview_load_failed_cb, task);
@@ -678,9 +645,9 @@ take_fresh_snapshot_in_background_if_stale (EphySnapshotService *service,
 }
 
 char *
-ephy_snapshot_service_get_snapshot_path_for_url_finish (EphySnapshotService *service,
-                                                        GAsyncResult        *result,
-                                                        GError             **error)
+ephy_snapshot_service_get_snapshot_path_for_url_finish (EphySnapshotService  *service,
+                                                        GAsyncResult         *result,
+                                                        GError              **error)
 {
   g_assert (g_task_is_valid (result, service));
 
@@ -692,6 +659,7 @@ got_snapshot_path_for_url (EphySnapshotService *service,
                            GAsyncResult        *result,
                            GTask               *task)
 {
+#ifndef __clang_analyzer__
   SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
@@ -703,6 +671,7 @@ got_snapshot_path_for_url (EphySnapshotService *service,
   } else {
     ephy_snapshot_service_take_from_webview (task);
   }
+#endif
 }
 
 void
@@ -743,9 +712,9 @@ ephy_snapshot_service_get_snapshot_path_async (EphySnapshotService *service,
 }
 
 char *
-ephy_snapshot_service_get_snapshot_path_finish (EphySnapshotService *service,
-                                                GAsyncResult        *result,
-                                                GError             **error)
+ephy_snapshot_service_get_snapshot_path_finish (EphySnapshotService  *service,
+                                                GAsyncResult         *result,
+                                                GError              **error)
 {
   g_assert (g_task_is_valid (result, service));
 
@@ -763,13 +732,15 @@ got_snapshot_path_to_delete_cb (EphySnapshotService *service,
   if (path)
     unlink (path);
   g_free (path);
+
+  g_object_unref (service);
 }
 
 void
 ephy_snapshot_service_delete_snapshot_for_url (EphySnapshotService *service,
                                                const char          *url)
 {
-  ephy_snapshot_service_get_snapshot_path_for_url_async (service,
+  ephy_snapshot_service_get_snapshot_path_for_url_async (g_object_ref (service),
                                                          url,
                                                          NULL,
                                                          (GAsyncReadyCallback)got_snapshot_path_to_delete_cb,
