@@ -113,14 +113,16 @@ GQuark ephy_sqlite_error_quark (void)
 }
 
 static void
-set_error_from_string (const char *string, GError **error)
+set_error_from_string (const char  *string,
+                       GError     **error)
 {
   if (error)
     *error = g_error_new_literal (ephy_sqlite_error_quark (), 0, string);
 }
 
 EphySQLiteConnection *
-ephy_sqlite_connection_new (EphySQLiteConnectionMode mode, const char *database_path)
+ephy_sqlite_connection_new (EphySQLiteConnectionMode  mode,
+                            const char               *database_path)
 {
   return EPHY_SQLITE_CONNECTION (g_object_new (EPHY_TYPE_SQLITE_CONNECTION,
                                                "mode", mode,
@@ -129,7 +131,8 @@ ephy_sqlite_connection_new (EphySQLiteConnectionMode mode, const char *database_
 }
 
 gboolean
-ephy_sqlite_connection_open (EphySQLiteConnection *self, GError **error)
+ephy_sqlite_connection_open (EphySQLiteConnection  *self,
+                             GError               **error)
 {
   if (self->database) {
     set_error_from_string ("Connection already open.", error);
@@ -138,14 +141,35 @@ ephy_sqlite_connection_open (EphySQLiteConnection *self, GError **error)
 
   if (sqlite3_open_v2 (self->database_path,
                        &self->database,
-                       self->mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY ? SQLITE_OPEN_READONLY
-                                                                           : SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE,
+                       self->mode == EPHY_SQLITE_CONNECTION_MODE_MEMORY ? SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE | SQLITE_OPEN_MEMORY
+                                                                        : SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE,
                        NULL) != SQLITE_OK) {
     ephy_sqlite_connection_get_error (self, error);
     self->database = NULL;
     return FALSE;
   }
 
+  /* Create a local copy of current database for in memory usage */
+  if (self->mode == EPHY_SQLITE_CONNECTION_MODE_MEMORY) {
+    sqlite3 *init_db;
+    int rc;
+
+    rc = sqlite3_open_v2 (self->database_path, &init_db, SQLITE_OPEN_READONLY, 0);
+    if (rc == SQLITE_OK) {
+      sqlite3_backup *backup;
+
+      backup = sqlite3_backup_init (self->database, "main", init_db, "main");
+      rc = sqlite3_backup_step (backup, -1);
+
+      if (rc != SQLITE_DONE)
+        g_warning ("Failed to copy history to in-memory database: %s", sqlite3_errstr (rc));
+
+      sqlite3_backup_finish (backup);
+    }
+
+    sqlite3_close (init_db);
+  }
+
   return TRUE;
 }
 
@@ -176,14 +200,17 @@ ephy_sqlite_connection_delete_database (EphySQLiteConnection *self)
 }
 
 void
-ephy_sqlite_connection_get_error (EphySQLiteConnection *self, GError **error)
+ephy_sqlite_connection_get_error (EphySQLiteConnection  *self,
+                                  GError               **error)
 {
   if (error)
     *error = g_error_new_literal (ephy_sqlite_error_quark (), sqlite3_errcode (self->database), sqlite3_errmsg (self->database));
 }
 
 gboolean
-ephy_sqlite_connection_execute (EphySQLiteConnection *self, const char *sql, GError **error)
+ephy_sqlite_connection_execute (EphySQLiteConnection  *self,
+                                const char            *sql,
+                                GError               **error)
 {
   if (self->database == NULL) {
     set_error_from_string ("Connection not open.", error);
@@ -198,7 +225,9 @@ ephy_sqlite_connection_execute (EphySQLiteConnection *self, const char *sql, GEr
 }
 
 EphySQLiteStatement *
-ephy_sqlite_connection_create_statement (EphySQLiteConnection *self, const char *sql, GError **error)
+ephy_sqlite_connection_create_statement (EphySQLiteConnection  *self,
+                                         const char            *sql,
+                                         GError               **error)
 {
   sqlite3_stmt *prepared_statement;
 
@@ -239,23 +268,22 @@ ephy_sqlite_connection_enable_foreign_keys (EphySQLiteConnection *self)
 }
 
 gboolean
-ephy_sqlite_connection_begin_transaction (EphySQLiteConnection *self, GError **error)
+ephy_sqlite_connection_begin_transaction (EphySQLiteConnection  *self,
+                                          GError               **error)
 {
-  if (self->mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY)
-    return TRUE;
   return ephy_sqlite_connection_execute (self, "BEGIN TRANSACTION", error);
 }
 
 gboolean
-ephy_sqlite_connection_commit_transaction (EphySQLiteConnection *self, GError **error)
+ephy_sqlite_connection_commit_transaction (EphySQLiteConnection  *self,
+                                           GError               **error)
 {
-  if (self->mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY)
-    return TRUE;
   return ephy_sqlite_connection_execute (self, "COMMIT", error);
 }
 
 gboolean
-ephy_sqlite_connection_table_exists (EphySQLiteConnection *self, const char *table_name)
+ephy_sqlite_connection_table_exists (EphySQLiteConnection *self,
+                                     const char           *table_name)
 {
   GError *error = NULL;
   gboolean table_exists = FALSE;
