@@ -116,16 +116,16 @@ ephy_web_application_get_program_name_from_profile_directory (const char *profil
 {
   const char *name;
 
-  // Just get the basename
+  /* Just get the basename */
   name = strrchr (profile_dir, G_DIR_SEPARATOR);
   if (name == NULL) {
     g_warning ("Profile directoroy %s is not a valid path", profile_dir);
     return NULL;
   }
 
-  name++; // Strip '/'
+  name++; /* Strip '/' */
 
-  // Legacy web app support
+  /* Legacy web app support */
   if (g_str_has_prefix (name, "app-"))
     name += strlen ("app-");
 
@@ -213,8 +213,8 @@ ephy_web_application_delete (const char *id)
   g_autofree char *config_dir = NULL;
   g_autofree char *desktop_file = NULL;
   g_autofree char *desktop_path = NULL;
-  g_autoptr(GFile) launcher = NULL;
-  g_autoptr(GError) error = NULL;
+  g_autoptr (GFile) launcher = NULL;
+  g_autoptr (GError) error = NULL;
 
   g_assert (id);
 
@@ -304,7 +304,7 @@ create_desktop_file (const char *id,
   g_key_file_set_value (file, "Desktop Entry", "StartupNotify", "true");
   g_key_file_set_value (file, "Desktop Entry", "Terminal", "false");
   g_key_file_set_value (file, "Desktop Entry", "Type", "Application");
-  g_key_file_set_value (file, "Desktop Entry", "Categories", "Network;GNOME;GTK;");
+  g_key_file_set_value (file, "Desktop Entry", "Categories", "GNOME;GTK;");
 
   if (icon) {
     GOutputStream *stream;
@@ -374,46 +374,43 @@ ephy_web_application_create (const char *id,
                              const char *name,
                              GdkPixbuf  *icon)
 {
-  char *profile_dir;
-  char *desktop_file_path = NULL;
+  g_autofree char *app_file = NULL;
+  g_autofree char *profile_dir = NULL;
+  g_autofree char *desktop_file_path = NULL;
+  int fd;
 
   /* If there's already a WebApp profile for the contents of this
    * view, do nothing. */
   profile_dir = ephy_web_application_get_profile_directory (id);
   if (g_file_test (profile_dir, G_FILE_TEST_IS_DIR)) {
     g_warning ("Profile directory %s already exists", profile_dir);
-    goto out;
+    return NULL;
   }
 
   /* Create the profile directory, populate it. */
   if (g_mkdir_with_parents (profile_dir, 488) == -1) {
     g_warning ("Failed to create directory %s", profile_dir);
-    goto out;
+    return NULL;
   }
 
   /* Skip migration for new web apps. */
   ephy_profile_utils_set_migration_version_for_profile_dir (EPHY_PROFILE_MIGRATION_VERSION, profile_dir);
 
   /* Create an .app file. */
-  g_autofree char *app_file = g_build_filename (profile_dir, ".app", NULL);
-  int fd = g_open (app_file, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+  app_file = g_build_filename (profile_dir, ".app", NULL);
+  fd = g_open (app_file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
   if (fd < 0) {
-    LOG ("Failed to create .app file: %s", g_strerror (errno));
-    goto out;
-  } else {
-    close (fd);
+    g_warning ("Failed to create .app file: %s", g_strerror (errno));
+    return NULL;
   }
+  close (fd);
 
   /* Create the deskop file. */
   desktop_file_path = create_desktop_file (id, name, address, profile_dir, icon);
   if (desktop_file_path)
     ephy_web_application_initialize_settings (profile_dir);
 
- out:
-  if (profile_dir)
-    g_free (profile_dir);
-
-  return desktop_file_path;
+  return g_steal_pointer (&desktop_file_path);
 }
 
 char *
@@ -511,7 +508,6 @@ ephy_web_application_setup_from_desktop_file (GDesktopAppInfo *desktop_info)
     if (names)
       gtk_window_set_default_icon_name (names[0]);
   }
-  g_clear_object (&icon);
 
   /* We need to re-set this because we have already parsed the
    * options, which inits GTK+ and sets this as a side effect.
@@ -550,7 +546,7 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
 
   id = get_app_id_from_profile_directory (profile_dir);
   if (!id)
-    return NULL;
+    g_error ("Cannot create EphyWebApplication: failed to get app ID from profile directory %s", profile_dir);
 
   app = g_new0 (EphyWebApplication, 1);
   app->id = g_strdup (id);
@@ -558,11 +554,8 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   app->desktop_file = get_app_desktop_filename (id);
   desktop_file_path = g_build_filename (profile_dir, app->desktop_file, NULL);
   desktop_info = g_desktop_app_info_new_from_filename (desktop_file_path);
-  if (!desktop_info) {
-    ephy_web_application_free (app);
-    g_free (desktop_file_path);
-    return NULL;
-  }
+  if (!desktop_info)
+    g_error ("Cannot create EphyWebApplication: failed to create GDesktopAppInfo from desktop file %s", desktop_file_path);
 
   app->name = g_strdup (g_app_info_get_name (G_APP_INFO (desktop_info)));
   app->icon_url = g_desktop_app_info_get_string (desktop_info, "Icon");
@@ -599,7 +592,7 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
   GFileInfo *info;
   GList *applications = NULL;
   g_autofree char *parent_directory_path = NULL;
-  g_autoptr(GFile) parent_directory = NULL;
+  g_autoptr (GFile) parent_directory = NULL;
 
   if (only_legacy)
     parent_directory_path = g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
@@ -631,7 +624,7 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
           if (g_file_test (app_file, G_FILE_TEST_EXISTS))
             applications = g_list_prepend (applications, app);
           else
-            g_object_unref (app);
+            g_free (app);
         } else
           applications = g_list_prepend (applications, app);
       }
@@ -759,14 +752,43 @@ ephy_web_application_initialize_settings (const char *profile_directory)
   g_free (name);
 }
 
-gboolean
-ephy_web_application_is_uri_allowed (const char* uri)
+static gboolean
+urls_have_same_origin (const char *a_url,
+                       const char *b_url)
 {
+  SoupURI *a_uri, *b_uri;
+  gboolean retval = FALSE;
+
+  a_uri = soup_uri_new (a_url);
+  if (!a_uri)
+    return retval;
+
+  b_uri = soup_uri_new (b_url);
+  if (b_uri) {
+    retval = a_uri->host && b_uri->host && soup_uri_host_equal (a_uri, b_uri);
+    soup_uri_free (b_uri);
+  }
+
+  soup_uri_free (a_uri);
+
+  return retval;
+}
+
+gboolean
+ephy_web_application_is_uri_allowed (const char *uri)
+{
+  EphyWebApplication *webapp = ephy_web_application_for_profile_directory (ephy_profile_dir ());
   SoupURI *request_uri;
   char **urls;
   guint i;
   gboolean matched = FALSE;
 
+  if (g_str_has_prefix (uri, "blob:") || g_str_has_prefix (uri, "data:"))
+    return TRUE;
+
+  if (urls_have_same_origin (uri, webapp->url))
+    return TRUE;
+
   if (g_strcmp0 (uri, "about:blank") == 0)
     return TRUE;
 
@@ -792,6 +814,18 @@ ephy_web_application_is_uri_allowed (const char* uri)
   return matched;
 }
 
+static void
+ephy_web_icon_copy_cb (GFile        *file,
+                       GAsyncResult *result,
+                       gpointer      user_data)
+{
+  GError *error = NULL;
+  if (!g_file_copy_finish (file, result, &error)) {
+    g_warning ("Failed to update web app icon: %s", error->message);
+    g_error_free (error);
+  }
+}
+
 gboolean
 ephy_web_application_save (EphyWebApplication *app)
 {
@@ -824,8 +858,14 @@ ephy_web_application_save (EphyWebApplication *app)
 
     icon = g_key_file_get_string (key, "Desktop Entry", "Icon", NULL);
     if (g_strcmp0 (icon, app->icon_url) != 0) {
+      GFile *new_icon;
+      GFile *old_icon;
       changed = TRUE;
-      g_key_file_set_string (key, "Desktop Entry", "Icon", app->icon_url);
+      new_icon = g_file_new_for_path (app->icon_url);
+      old_icon = g_file_new_for_path (icon);
+      g_file_copy_async (new_icon, old_icon, G_FILE_COPY_OVERWRITE,
+                         G_PRIORITY_DEFAULT, NULL, NULL, NULL,
+                         (GAsyncReadyCallback)ephy_web_icon_copy_cb, NULL);
     }
     g_free (icon);
 
