@@ -40,18 +40,18 @@
 struct _EphyGSBService {
   GObject parent_instance;
 
-  char           *api_key;
+  char *api_key;
   EphyGSBStorage *storage;
 
-  gboolean        is_updating;
-  guint           source_id;
+  gboolean is_updating;
+  guint source_id;
 
-  gint64          next_full_hashes_time;
-  gint64          next_list_updates_time;
-  gint64          back_off_exit_time;
-  gint64          back_off_num_fails;
+  gint64 next_full_hashes_time;
+  gint64 next_list_updates_time;
+  gint64 back_off_exit_time;
+  gint64 back_off_num_fails;
 
-  SoupSession    *session;
+  SoupSession *session;
 };
 
 G_DEFINE_TYPE (EphyGSBService, ephy_gsb_service, G_TYPE_OBJECT);
@@ -335,7 +335,7 @@ ephy_gsb_service_set_property (GObject      *object,
   switch (prop_id) {
     case PROP_API_KEY:
       g_free (self->api_key);
-      self->api_key = g_strdup (g_value_get_string (value));
+      self->api_key = g_value_dup_string (value);
       break;
     case PROP_GSB_STORAGE:
       if (self->storage)
@@ -385,10 +385,7 @@ ephy_gsb_service_dispose (GObject *object)
   g_clear_object (&self->storage);
   g_clear_object (&self->session);
 
-  if (self->source_id != 0) {
-    g_source_remove (self->source_id);
-    self->source_id = 0;
-  }
+  g_clear_handle_id (&self->source_id, g_source_remove);
 
   G_OBJECT_CLASS (ephy_gsb_service_parent_class)->dispose (object);
 }
@@ -436,7 +433,7 @@ static void
 ephy_gsb_service_init (EphyGSBService *self)
 {
   self->session = soup_session_new ();
-  g_object_set (self->session, "user-agent", ephy_user_agent_get_internal (), NULL);
+  g_object_set (self->session, "user-agent", ephy_user_agent_get (), NULL);
 }
 
 static void
