@@ -543,18 +543,18 @@ ephy_sync_crypto_hkdf (const guint8 *in,
   prk = g_malloc (SHA256_DIGEST_SIZE);
   out = g_malloc (out_len);
 
-  hmac_sha256_set_key(&ctx, SHA256_DIGEST_SIZE, salt);
-  hkdf_extract(&ctx,
+  hmac_sha256_set_key (&ctx, SHA256_DIGEST_SIZE, salt);
+  hkdf_extract (&ctx,
+                (nettle_hash_update_func *)hmac_sha256_update,
+                (nettle_hash_digest_func *)hmac_sha256_digest,
+                SHA256_DIGEST_SIZE,
+                in_len, in, prk);
+  hmac_sha256_set_key (&ctx, SHA256_DIGEST_SIZE, prk);
+  hkdf_expand (&ctx,
                (nettle_hash_update_func *)hmac_sha256_update,
                (nettle_hash_digest_func *)hmac_sha256_digest,
                SHA256_DIGEST_SIZE,
-               in_len, in, prk);
-  hmac_sha256_set_key(&ctx, SHA256_DIGEST_SIZE, prk);
-  hkdf_expand(&ctx,
-              (nettle_hash_update_func *)hmac_sha256_update,
-              (nettle_hash_digest_func *)hmac_sha256_digest,
-              SHA256_DIGEST_SIZE,
-              info_len, info, out_len, out);
+               info_len, info, out_len, out);
 
   g_free (salt);
   g_free (prk);
@@ -693,15 +693,15 @@ ephy_sync_crypto_derive_master_keys (const char    *bundle_hex,
                                      guint8       **ka,
                                      guint8       **kb)
 {
+  gsize len = 32; /* The master sync keys are always 32 bytes. */
   guint8 *bundle;
-  guint8 *ciphertext;
-  guint8 *resp_hmac;
+  guint8 ciphertext[2 * len];
+  guint8 resp_hmac[len];
   guint8 *resp_hmac_2;
   guint8 *xored;
   guint8 *wrap_kb;
   char *resp_hmac_2_hex;
   gboolean retval = TRUE;
-  gsize len = 32; /* The master sync keys are always 32 bytes. */
 
   g_assert (bundle_hex);
   g_assert (resp_hmac_key);
@@ -711,8 +711,6 @@ ephy_sync_crypto_derive_master_keys (const char    *bundle_hex,
   g_assert (kb);
 
   bundle = ephy_sync_utils_decode_hex (bundle_hex);
-  ciphertext = g_malloc (2 * len);
-  resp_hmac = g_malloc (len);
 
   /* Compute the MAC and compare it to the expected value. */
   memcpy (ciphertext, bundle, 2 * len);
@@ -743,8 +741,6 @@ ephy_sync_crypto_derive_master_keys (const char    *bundle_hex,
 out:
   g_free (resp_hmac_2);
   g_free (resp_hmac_2_hex);
-  g_free (resp_hmac);
-  g_free (ciphertext);
   g_free (bundle);
 
   return retval;
@@ -874,7 +870,7 @@ ephy_sync_crypto_create_assertion (const char           *certificate,
 
   /* Encode the header and body to base64 url safe and join them. */
   expires_at = g_get_real_time () / 1000 + seconds * 1000;
-  body = g_strdup_printf ("{\"exp\": %"PRIu64", \"aud\": \"%s\"}", expires_at, audience);
+  body = g_strdup_printf ("{\"exp\": %" PRIu64 ", \"aud\": \"%s\"}", expires_at, audience);
   body_b64 = ephy_sync_utils_base64_urlsafe_encode ((guint8 *)body, strlen (body), TRUE);
   header_b64 = ephy_sync_utils_base64_urlsafe_encode ((guint8 *)header, strlen (header), TRUE);
   to_sign = g_strdup_printf ("%s.%s", header_b64, body_b64);
@@ -942,7 +938,7 @@ ephy_sync_crypto_aes_256_encrypt (const char   *text,
   guint8 *padded;
   guint8 *encrypted;
   gsize padded_len;
-  struct CBC_CTX(struct aes256_ctx, AES_BLOCK_SIZE) ctx;
+  struct CBC_CTX (struct aes256_ctx, AES_BLOCK_SIZE) ctx;
 
   g_assert (text);
   g_assert (key);
@@ -952,9 +948,9 @@ ephy_sync_crypto_aes_256_encrypt (const char   *text,
   padded = ephy_sync_crypto_pad (text, AES_BLOCK_SIZE, &padded_len);
   encrypted = g_malloc (padded_len);
 
-  aes256_set_encrypt_key(&ctx.ctx, key);
-  CBC_SET_IV(&ctx, iv);
-  CBC_ENCRYPT(&ctx, aes256_encrypt, padded_len, encrypted, padded);
+  aes256_set_encrypt_key (&ctx.ctx, key);
+  CBC_SET_IV (&ctx, iv);
+  CBC_ENCRYPT (&ctx, aes256_encrypt, padded_len, encrypted, padded);
 
   *out_len = padded_len;
   g_free (padded);
@@ -1070,7 +1066,7 @@ ephy_sync_crypto_aes_256_decrypt (const guint8 *data,
 {
   guint8 *decrypted;
   char *unpadded;
-  struct CBC_CTX(struct aes256_ctx, AES_BLOCK_SIZE) ctx;
+  struct CBC_CTX (struct aes256_ctx, AES_BLOCK_SIZE) ctx;
 
   g_assert (data);
   g_assert (key);
