@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ */
 
 #include "config.h"
 
@@ -33,24 +33,25 @@
 #include <string.h>
 
 struct _EphyBookmarkPropertiesGrid {
-  GtkGrid                         parent_instance;
+  GtkGrid parent_instance;
 
-  EphyBookmarksManager           *manager;
-  EphyBookmark                   *bookmark;
-  gboolean                        bookmark_is_modified;
-  gboolean                        bookmark_is_removed;
+  EphyBookmarksManager *manager;
+  EphyBookmark *bookmark;
+  gboolean bookmark_is_modified;
+  gboolean bookmark_is_removed;
 
-  EphyBookmarkPropertiesGridType  type;
-  GtkWidget                      *parent;
+  EphyBookmarkPropertiesGridType type;
+  GtkWidget *parent;
 
-  GtkWidget                      *popover_bookmark_label;
-  GtkWidget                      *name_entry;
-  GtkWidget                      *address_entry;
-  GtkWidget                      *popover_tags_label;
-  GtkWidget                      *tags_box;
-  GtkWidget                      *add_tag_entry;
-  GtkWidget                      *add_tag_button;
-  GtkWidget                      *remove_bookmark_button;
+  GtkWidget *popover_bookmark_label;
+  GtkWidget *name_entry;
+  GtkWidget *address_label;
+  GtkWidget *address_entry;
+  GtkWidget *popover_tags_label;
+  GtkWidget *tags_box;
+  GtkWidget *tags_scrolled_window;
+  GtkWidget *add_tag_entry;
+  GtkWidget *add_tag_button;
 };
 
 G_DEFINE_TYPE (EphyBookmarkPropertiesGrid, ephy_bookmark_properties_grid, GTK_TYPE_GRID)
@@ -66,7 +67,8 @@ enum {
 static GParamSpec *obj_properties[LAST_PROP];
 
 static int
-flow_box_sort_func (GtkFlowBoxChild *child1, GtkFlowBoxChild *child2)
+flow_box_sort_func (GtkFlowBoxChild *child1,
+                    GtkFlowBoxChild *child2)
 {
   GtkWidget *box1;
   GtkWidget *box2;
@@ -90,6 +92,21 @@ flow_box_sort_func (GtkFlowBoxChild *child1, GtkFlowBoxChild *child2)
   return ephy_bookmark_tags_compare (tag1, tag2);
 }
 
+static void
+update_tags_scrollbar (EphyBookmarkPropertiesGrid *self)
+{
+  g_autoptr (GList) children = NULL;
+  gint n_tags;
+
+  children = gtk_container_get_children (GTK_CONTAINER (self->tags_box));
+  n_tags = g_list_length (children);
+
+  g_object_set (self->tags_scrolled_window,
+                "vscrollbar-policy",
+                (n_tags > 3) ? GTK_POLICY_AUTOMATIC : GTK_POLICY_NEVER,
+                NULL);
+}
+
 static void
 ephy_bookmark_properties_grid_tags_box_child_activated_cb (EphyBookmarkPropertiesGrid *self,
                                                            GtkFlowBoxChild            *child,
@@ -137,6 +154,7 @@ ephy_bookmark_properties_grid_tag_widget_button_clicked_cb (EphyBookmarkProperti
 
   flow_box_child = gtk_widget_get_parent (box);
   gtk_widget_destroy (flow_box_child);
+  update_tags_scrollbar (self);
 }
 
 static GtkWidget *
@@ -168,8 +186,9 @@ ephy_bookmark_properties_grid_create_tag_widget (EphyBookmarkPropertiesGrid *sel
     gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
   }
 
-  label_text = default_tag ? _(EPHY_BOOKMARKS_FAVORITES_TAG) : tag;
+  label_text = default_tag ? EPHY_BOOKMARKS_FAVORITES_TAG : tag;
   label = gtk_label_new (label_text);
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
   gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
   if (!default_tag) {
@@ -226,6 +245,7 @@ ephy_bookmarks_properties_grid_actions_add_tag (GSimpleAction *action,
   /* Create a new widget for the new tag */
   widget = ephy_bookmark_properties_grid_create_tag_widget (self, text, TRUE);
   gtk_flow_box_insert (GTK_FLOW_BOX (self->tags_box), widget, -1);
+  update_tags_scrollbar (self);
 
   /* Empty entry and disable button's action until new text is inserted */
   gtk_entry_set_text (GTK_ENTRY (self->add_tag_entry), "");
@@ -359,9 +379,8 @@ ephy_bookmark_properties_grid_constructed (GObject *object)
   /* Set appearance based on type */
   if (self->type == EPHY_BOOKMARK_PROPERTIES_GRID_TYPE_DIALOG) {
     gtk_container_remove (GTK_CONTAINER (self), self->popover_bookmark_label);
-    gtk_container_remove (GTK_CONTAINER (self), self->popover_tags_label);
   } else if (self->type == EPHY_BOOKMARK_PROPERTIES_GRID_TYPE_POPOVER) {
-    gtk_grid_remove_column (GTK_GRID (self), 0);
+    gtk_container_remove (GTK_CONTAINER (self), self->address_label);
     gtk_container_remove (GTK_CONTAINER (self), self->address_entry);
   }
 
@@ -405,6 +424,7 @@ ephy_bookmark_properties_grid_constructed (GObject *object)
     widget = ephy_bookmark_properties_grid_create_tag_widget (self, tag, selected);
     gtk_flow_box_insert (GTK_FLOW_BOX (self->tags_box), widget, -1);
   }
+  update_tags_scrollbar (self);
 
   g_signal_connect_object (self->tags_box, "child-activated",
                            G_CALLBACK (ephy_bookmark_properties_grid_tags_box_child_activated_cb),
@@ -421,9 +441,10 @@ ephy_bookmark_properties_grid_finalize (GObject *object)
   if (self->bookmark_is_modified && !self->bookmark_is_removed)
     g_signal_emit_by_name (self->manager, "synchronizable-modified", self->bookmark, FALSE);
 
-  ephy_bookmarks_manager_save_to_file_async (self->manager, NULL,
-                                             ephy_bookmarks_manager_save_to_file_warn_on_error_cb,
-                                             NULL);
+  ephy_bookmarks_manager_save (self->manager,
+                               ephy_bookmarks_manager_save_warn_on_error_cancellable (self->manager),
+                               ephy_bookmarks_manager_save_warn_on_error_cb,
+                               NULL);
 
   G_OBJECT_CLASS (ephy_bookmark_properties_grid_parent_class)->finalize (object);
 }
@@ -465,12 +486,13 @@ ephy_bookmark_properties_grid_class_init (EphyBookmarkPropertiesGridClass *klass
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/epiphany/gtk/bookmark-properties-grid.ui");
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, popover_bookmark_label);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, name_entry);
+  gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, address_label);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, address_entry);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, popover_tags_label);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, tags_box);
+  gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, tags_scrolled_window);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, add_tag_entry);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, add_tag_button);
-  gtk_widget_class_bind_template_child (widget_class, EphyBookmarkPropertiesGrid, remove_bookmark_button);
 }
 
 static const GActionEntry entries[] = {
@@ -481,7 +503,7 @@ static const GActionEntry entries[] = {
 static void
 ephy_bookmark_properties_grid_init (EphyBookmarkPropertiesGrid *self)
 {
-  g_autoptr(GSimpleActionGroup) group = NULL;
+  g_autoptr (GSimpleActionGroup) group = NULL;
   GAction *action;
 
   gtk_widget_init_template (GTK_WIDGET (self));
