@@ -22,7 +22,9 @@
 #include "ephy-bookmark.h"
 
 #include "ephy-bookmarks-manager.h"
+#include "ephy-shell.h"
 #include "ephy-synchronizable.h"
+#include "ephy-sync-utils.h"
 
 #include <string.h>
 
@@ -32,30 +34,30 @@
 #define BOOKMARK_LOAD_IN_SIDEBAR_VAL FALSE
 
 struct _EphyBookmark {
-  GObject      parent_instance;
+  GObject parent_instance;
 
-  char        *url;
-  char        *title;
-  GSequence   *tags;
-  gint64       time_added;
+  char *url;
+  char *title;
+  GSequence *tags;
+  gint64 time_added;
 
   /* Firefox Sync specific fields. */
-  char        *id;
-  char        *type;
-  char        *parent_id;
-  char        *parent_name;
-  gboolean     load_in_sidebar;
-  gint64       server_time_modified;
+  char *id;
+  char *type;
+  char *parent_id;
+  char *parent_name;
+  gboolean load_in_sidebar;
+  gint64 server_time_modified;
 };
 
 static void json_serializable_iface_init (JsonSerializableIface *iface);
 static void ephy_synchronizable_iface_init (EphySynchronizableInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (EphyBookmark, ephy_bookmark, G_TYPE_OBJECT,
-                        G_IMPLEMENT_INTERFACE (JSON_TYPE_SERIALIZABLE,
-                                               json_serializable_iface_init)
-                        G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE,
-                                               ephy_synchronizable_iface_init))
+                         G_IMPLEMENT_INTERFACE (JSON_TYPE_SERIALIZABLE,
+                                                json_serializable_iface_init)
+                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_SYNCHRONIZABLE,
+                                                ephy_synchronizable_iface_init))
 
 enum {
   PROP_0,
@@ -78,7 +80,7 @@ enum {
 };
 
 static GParamSpec *obj_properties[LAST_PROP];
-static guint       signals[LAST_SIGNAL];
+static guint signals[LAST_SIGNAL];
 
 static void
 ephy_bookmark_set_property (GObject      *object,
@@ -175,10 +177,14 @@ ephy_bookmark_finalize (GObject *object)
 
   g_free (self->url);
   g_free (self->title);
-  g_free (self->id);
 
   g_sequence_free (self->tags);
 
+  g_free (self->id);
+  g_free (self->type);
+  g_free (self->parent_id);
+  g_free (self->parent_name);
+
   G_OBJECT_CLASS (ephy_bookmark_parent_class)->finalize (object);
 }
 
@@ -306,9 +312,8 @@ ephy_bookmark_set_time_added (EphyBookmark *self,
                               gint64        time_added)
 {
   g_assert (EPHY_IS_BOOKMARK (self));
-  g_assert (time_added >= 0);
 
-  self->time_added = time_added;
+  self->time_added = (time_added >= 0 ? time_added : g_get_real_time ());
 }
 
 gint64
@@ -321,7 +326,8 @@ ephy_bookmark_get_time_added (EphyBookmark *self)
 
 
 void
-ephy_bookmark_set_url (EphyBookmark *self, const char *url)
+ephy_bookmark_set_url (EphyBookmark *self,
+                       const char   *url)
 {
   g_assert (EPHY_IS_BOOKMARK (self));
 
@@ -338,7 +344,8 @@ ephy_bookmark_get_url (EphyBookmark *self)
 }
 
 void
-ephy_bookmark_set_title (EphyBookmark *self, const char *title)
+ephy_bookmark_set_title (EphyBookmark *self,
+                         const char   *title)
 {
   g_assert (EPHY_IS_BOOKMARK (self));
 
@@ -378,7 +385,6 @@ void
 ephy_bookmark_set_is_uploaded (EphyBookmark *self,
                                gboolean      uploaded)
 {
-
   /* FIXME: This is no longer used for Firefox Sync, but bookmarks import/export
    * expects it. We need to delete it and write a migrator for bookmarks. */
   g_assert (EPHY_IS_BOOKMARK (self));
@@ -438,7 +444,8 @@ ephy_bookmark_remove_tag (EphyBookmark *self,
 }
 
 gboolean
-ephy_bookmark_has_tag (EphyBookmark *self, const char *tag)
+ephy_bookmark_has_tag (EphyBookmark *self,
+                       const char   *tag)
 {
   GSequenceIter *tag_iter;
 
@@ -495,7 +502,8 @@ ephy_bookmark_bookmarks_compare_func (EphyBookmark *bookmark1,
 }
 
 int
-ephy_bookmark_tags_compare (const char *tag1, const char *tag2)
+ephy_bookmark_tags_compare (const char *tag1,
+                            const char *tag2)
 {
   int result;
 
@@ -515,6 +523,25 @@ ephy_bookmark_tags_compare (const char *tag1, const char *tag2)
   return result;
 }
 
+char *
+ephy_bookmark_generate_random_id (void)
+{
+  char *id = NULL;
+  EphyBookmarksManager *manager;
+
+  manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+
+  while (!id) {
+    id = ephy_sync_utils_get_random_sync_id ();
+
+    /* Check if the generated id isn't used already. */
+    if (ephy_bookmarks_manager_get_bookmark_by_id (manager, id))
+      g_clear_pointer (&id, g_free);
+  }
+
+  return id;
+}
+
 static JsonNode *
 serializable_serialize_property (JsonSerializable *serializable,
                                  const char       *name,
