@@ -24,9 +24,11 @@
 #include "ephy-header-bar.h"
 
 #include "ephy-add-bookmark-popover.h"
+#include "ephy-desktop-utils.h"
 #include "ephy-embed-utils.h"
 #include "ephy-flatpak-utils.h"
 #include "ephy-location-entry.h"
+#include "ephy-notebook.h"
 #include "ephy-shell.h"
 #include "ephy-title-box.h"
 #include "ephy-title-widget.h"
@@ -53,13 +55,9 @@ struct _EphyHeaderBar {
   GtkRevealer *end_revealer;
   EphyActionBarStart *action_bar_start;
   EphyActionBarEnd *action_bar_end;
-  GtkWidget *navigation_box;
-  GtkWidget *reader_mode_revealer;
-  GtkWidget *reader_mode_button;
-  GtkWidget *new_tab_button;
-  GtkWidget *bookmarks_button;
   GtkWidget *page_menu_button;
   GtkWidget *zoom_level_button;
+  GtkWidget *restore_button;
 };
 
 G_DEFINE_TYPE (EphyHeaderBar, ephy_header_bar, GTK_TYPE_HEADER_BAR)
@@ -113,6 +111,18 @@ sync_chromes_visibility (EphyHeaderBar *header_bar)
   gtk_widget_set_visible (header_bar->page_menu_button, chrome & EPHY_WINDOW_CHROME_MENU);
 }
 
+void
+fullscreen_changed_cb (EphyHeaderBar *header_bar)
+{
+  gboolean fullscreen;
+
+  g_object_get (header_bar->window, "fullscreen", &fullscreen, NULL);
+
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (header_bar), !fullscreen);
+  gtk_widget_set_visible (header_bar->restore_button, fullscreen);
+  ephy_action_bar_set_is_fullscreen (header_bar->action_bar_start, fullscreen);
+}
+
 static void
 add_bookmark_button_clicked_cb (EphyLocationEntry *entry,
                                 gpointer          *user_data)
@@ -127,6 +137,19 @@ add_bookmark_button_clicked_cb (EphyLocationEntry *entry,
   g_action_activate (action, NULL);
 }
 
+static void
+restore_button_clicked_cb (GtkButton     *button,
+                           EphyHeaderBar *header_bar)
+{
+  GActionGroup *action_group;
+  GAction *action;
+
+  action_group = gtk_widget_get_action_group (GTK_WIDGET (header_bar->window), "win");
+  action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "fullscreen");
+
+  g_action_activate (action, NULL);
+}
+
 static void
 update_revealer_visibility (GtkRevealer *revealer)
 {
@@ -135,15 +158,34 @@ update_revealer_visibility (GtkRevealer *revealer)
                           gtk_revealer_get_child_revealed (revealer));
 }
 
-static gboolean
-is_desktop_pantheon (void)
+static void
+handle_primary_tab_key (GtkWidget *widget,
+                        gpointer   user_data)
 {
-  const gchar *xdg_current_desktop = g_environ_getenv (g_get_environ (), "XDG_CURRENT_DESKTOP");
+  EphyHeaderBar *header_bar = EPHY_HEADER_BAR (user_data);
+  GtkWidget *entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
+  GtkWidget *nb;
 
-  if (!xdg_current_desktop)
-      return FALSE;
+  nb = ephy_window_get_notebook (header_bar->window);
+  g_assert (nb != NULL);
 
-  return strstr (xdg_current_desktop, "Pantheon") != NULL;
+  ephy_notebook_next_page (EPHY_NOTEBOOK (nb));
+  gtk_widget_grab_focus (entry);
+}
+
+static void
+handle_primary_iso_tab_key (GtkWidget *widget,
+                            gpointer   user_data)
+{
+  EphyHeaderBar *header_bar = EPHY_HEADER_BAR (user_data);
+  GtkWidget *entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
+  GtkWidget *nb;
+
+  nb = ephy_window_get_notebook (header_bar->window);
+  g_assert (nb != NULL);
+
+  ephy_notebook_prev_page (EPHY_NOTEBOOK (nb));
+  gtk_widget_grab_focus (entry);
 }
 
 static void
@@ -154,13 +196,16 @@ ephy_header_bar_constructed (GObject *object)
   GtkWidget *page_menu_popover;
   GtkBuilder *builder;
   EphyEmbedShell *embed_shell;
-  HdyColumn *column;
+  GtkSizeGroup *downloads_size_group;
 
   G_OBJECT_CLASS (ephy_header_bar_parent_class)->constructed (object);
 
   g_signal_connect_object (header_bar->window, "notify::chrome",
                            G_CALLBACK (sync_chromes_visibility), header_bar,
                            G_CONNECT_SWAPPED);
+  g_signal_connect_object (header_bar->window, "notify::fullscreen",
+                           G_CALLBACK (fullscreen_changed_cb), header_bar,
+                           G_CONNECT_SWAPPED);
 
   /* Start action elements */
   header_bar->action_bar_start = ephy_action_bar_start_new ();
@@ -181,15 +226,51 @@ ephy_header_bar_constructed (GObject *object)
   /* Title widget (location entry or title box) */
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_title_box_new ());
-  else
+  else {
+    DzlShortcutController *controller;
+    GtkWidget *entry;
+
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_location_entry_new ());
-  column = hdy_column_new ();
-  gtk_widget_set_hexpand (GTK_WIDGET (column), TRUE);
-  gtk_widget_show (GTK_WIDGET (column));
-  hdy_column_set_maximum_width (column, 860);
-  hdy_column_set_linear_growth_width (column, 560);
-  gtk_container_add (GTK_CONTAINER (column), GTK_WIDGET (header_bar->title_widget));
-  gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (column));
+    entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
+    controller = dzl_shortcut_controller_find (entry);
+
+    dzl_shortcut_controller_add_command_callback (controller,
+                                                  "org.gnome.Epiphany.next-tab-pages",
+                                                  "<Primary>Tab",
+                                                  DZL_SHORTCUT_PHASE_DISPATCH,
+                                                  handle_primary_tab_key,
+                                                  header_bar,
+                                                  NULL);
+
+    dzl_shortcut_controller_add_command_callback (controller,
+                                                  "org.gnome.Epiphany.prev-tab-pages",
+                                                  "<Primary>ISO_Left_Tab",
+                                                  DZL_SHORTCUT_PHASE_DISPATCH,
+                                                  handle_primary_iso_tab_key,
+                                                  header_bar,
+                                                  NULL);
+  }
+
+  if (is_desktop_pantheon ()) {
+    /* Use a full-width entry on Pantheon */
+    gtk_widget_set_hexpand (GTK_WIDGET (header_bar->title_widget), TRUE);
+    gtk_widget_set_margin_start (GTK_WIDGET (header_bar->title_widget), 6);
+    gtk_widget_set_margin_end (GTK_WIDGET (header_bar->title_widget), 6);
+
+    gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (header_bar->title_widget));
+  } else {
+    HdyColumn *column;
+
+    column = hdy_column_new ();
+    gtk_widget_set_hexpand (GTK_WIDGET (column), TRUE);
+    gtk_widget_show (GTK_WIDGET (column));
+    hdy_column_set_maximum_width (column, 860);
+    hdy_column_set_linear_growth_width (column, 560);
+    gtk_container_add (GTK_CONTAINER (column), GTK_WIDGET (header_bar->title_widget));
+
+    gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (column));
+  }
+
   gtk_widget_show (GTK_WIDGET (header_bar->title_widget));
 
   if (EPHY_IS_LOCATION_ENTRY (header_bar->title_widget)) {
@@ -203,6 +284,16 @@ ephy_header_bar_constructed (GObject *object)
                              0);
   }
 
+  /* Fullscreen restore button */
+  header_bar->restore_button = gtk_button_new_from_icon_name ("view-restore-symbolic",
+                                                              GTK_ICON_SIZE_BUTTON);
+  gtk_widget_set_valign (header_bar->restore_button, GTK_ALIGN_CENTER);
+  g_signal_connect_object (header_bar->restore_button, "clicked",
+                           G_CALLBACK (restore_button_clicked_cb),
+                           header_bar, 0);
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (header_bar),
+                           GTK_WIDGET (header_bar->restore_button));
+
   /* Page Menu */
   button = gtk_menu_button_new ();
   header_bar->page_menu_button = button;
@@ -237,9 +328,14 @@ ephy_header_bar_constructed (GObject *object)
       gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "help-button")));
   }
 
-  if (is_desktop_pantheon ())
+  if (is_desktop_pantheon ()) {
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "about-button")));
 
+    gtk_button_set_image (GTK_BUTTON (button),
+                          gtk_image_new_from_icon_name ("open-menu",
+                                                        GTK_ICON_SIZE_LARGE_TOOLBAR));
+  }
+
   gtk_menu_button_set_popover (GTK_MENU_BUTTON (button), page_menu_popover);
   g_object_unref (builder);
 
@@ -258,6 +354,14 @@ ephy_header_bar_constructed (GObject *object)
 
   gtk_header_bar_pack_end (GTK_HEADER_BAR (header_bar),
                            GTK_WIDGET (header_bar->end_revealer));
+
+  /* Sync the size of placeholder in EphyActionBarStart with downloads button */
+  downloads_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+  gtk_size_group_add_widget (downloads_size_group,
+                             ephy_action_bar_start_get_placeholder (header_bar->action_bar_start));
+  gtk_size_group_add_widget (downloads_size_group,
+                             ephy_action_bar_end_get_downloads_revealer (header_bar->action_bar_end));
+  g_object_unref (downloads_size_group);
 }
 
 static void
@@ -338,15 +442,18 @@ ephy_header_bar_set_adaptive_mode (EphyHeaderBar    *header_bar,
                                    EphyAdaptiveMode  adaptive_mode)
 {
   switch (adaptive_mode) {
-  case EPHY_ADAPTIVE_MODE_NORMAL:
-    gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), TRUE);
-    gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), TRUE);
+    case EPHY_ADAPTIVE_MODE_NORMAL:
+      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), TRUE);
+      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), TRUE);
 
-    break;
-  case EPHY_ADAPTIVE_MODE_NARROW:
-    gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), FALSE);
-    gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), FALSE);
+      break;
+    case EPHY_ADAPTIVE_MODE_NARROW:
+      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), FALSE);
+      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), FALSE);
 
-    break;
+      break;
   }
+
+  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_APPLICATION)
+    ephy_location_entry_set_mobile_popdown (EPHY_LOCATION_ENTRY (header_bar->title_widget), adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW);
 }
