@@ -56,7 +56,6 @@ struct _EphyLocationController {
   gboolean editable;
   gboolean sync_address_is_blocked;
   EphySearchEngineManager *search_engine_manager;
-  guint num_search_engines_actions;
 };
 
 static void ephy_location_controller_finalize (GObject *object);
@@ -81,12 +80,13 @@ G_DEFINE_TYPE_WITH_CODE (EphyLocationController, ephy_location_controller, G_TYP
                                                 NULL))
 
 static void
-entry_drag_data_received_cb (GtkWidget *widget,
-                             GdkDragContext *context,
-                             gint x, gint y,
-                             GtkSelectionData *selection_data,
-                             guint info,
-                             guint time,
+entry_drag_data_received_cb (GtkWidget              *widget,
+                             GdkDragContext         *context,
+                             gint                    x,
+                             gint                    y,
+                             GtkSelectionData       *selection_data,
+                             guint                   info,
+                             guint                   time,
                              EphyLocationController *controller)
 {
   GtkEntry *entry;
@@ -146,6 +146,23 @@ entry_activate_cb (GtkEntry               *entry,
   if (content == NULL || content[0] == '\0')
     return;
 
+  if (g_str_has_prefix (content, "ephy-tab://")) {
+    GtkWidget *notebook = ephy_window_get_notebook (controller->window);
+    GtkWidget *tab;
+    EphyWebView *webview;
+    gint current = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+    guint64 page = g_ascii_strtoull (content + strlen ("ephy-tab://"), NULL, 0);
+
+    tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), current);
+    gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), page);
+    webview = ephy_embed_get_web_view (EPHY_EMBED (tab));
+
+    if (ephy_web_view_is_overview (webview))
+      g_signal_emit_by_name (GTK_NOTEBOOK (notebook), "tab-close-request", tab);
+
+    return;
+  }
+
   address = g_strdup (content);
   effective_address = ephy_embed_utils_normalize_or_autosearch_address (g_strstrip (address));
   g_free (address);
@@ -174,7 +191,8 @@ entry_activate_cb (GtkEntry               *entry,
 
 
 static void
-user_changed_cb (GtkWidget *widget, EphyLocationController *controller)
+user_changed_cb (GtkWidget              *widget,
+                 EphyLocationController *controller)
 {
   const char *address;
   DzlSuggestionEntry *entry = DZL_SUGGESTION_ENTRY (ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (widget)));
