@@ -35,13 +35,11 @@ struct _EphySearchEngineDialog {
   GtkDialog parent_instance;
 
   EphySearchEngineManager *search_engine_manager;
-  GtkWidget *search_engine_add_button;
   GtkWidget *search_engine_address_entry;
   GtkWidget *search_engine_default_switch;
   GtkWidget *search_engine_list_box;
   GtkWidget *search_engine_name_entry;
   GtkWidget *search_engine_bang_entry;
-  GtkWidget *search_engine_remove_button;
 };
 
 G_DEFINE_TYPE (EphySearchEngineDialog, ephy_search_engine_dialog, GTK_TYPE_DIALOG)
@@ -148,9 +146,9 @@ dialog_check_name_entry (const char             *name,
 
   /* Check if name already exist in the search_engine_manager */
   if (ephy_search_engine_manager_get_address (manager, name) != NULL &&
-      g_strcmp0 (name, search_engine_name) != 0 ) {
-      dialog_set_entry_error_state (dialog->search_engine_name_entry);
-      return FALSE;
+      g_strcmp0 (name, search_engine_name) != 0) {
+    dialog_set_entry_error_state (dialog->search_engine_name_entry);
+    return FALSE;
   }
 
   dialog_set_entry_normal_state (dialog->search_engine_name_entry);
@@ -164,8 +162,7 @@ dialog_check_address_entry (const char             *address,
   if (address == NULL || !soup_uri_new (address) || strstr (address, "%s") == NULL) {
     dialog_set_entry_error_state (dialog->search_engine_address_entry);
     return FALSE;
-  }
-  else
+  } else
     dialog_set_entry_normal_state (dialog->search_engine_address_entry);
 
   return TRUE;
@@ -274,7 +271,7 @@ ephy_search_engine_dialog_fill_list_box (EphySearchEngineDialog *dialog)
   GtkListBox *listbox;
   GtkWidget *list_box_row;
   EphySearchEngineManager *manager;
-  char **engines_names ;
+  char **engines_names;
 
   listbox = GTK_LIST_BOX (dialog->search_engine_list_box);
   manager = dialog->search_engine_manager;
@@ -299,7 +296,7 @@ static char *
 generate_new_unique_default_engine_name (EphySearchEngineDialog *dialog)
 {
   guint i = 1;
-  char *default_name = g_strdup_printf ("%s %d", NEW_SEARCH_ENGINE_NAME, i);
+  char *default_name = g_strdup_printf ("%s %u", NEW_SEARCH_ENGINE_NAME, i);
 
   while (dialog_list_box_child_n_occurence (default_name, dialog) != 0) {
     if (i == UINT_MAX)
@@ -307,7 +304,7 @@ generate_new_unique_default_engine_name (EphySearchEngineDialog *dialog)
 
     i++;
     g_free (default_name);
-    default_name = g_strdup_printf ("%s %d", NEW_SEARCH_ENGINE_NAME, i);
+    default_name = g_strdup_printf ("%s %u", NEW_SEARCH_ENGINE_NAME, i);
   }
 
   return default_name;
@@ -359,13 +356,13 @@ on_search_engine_remove_button_clicked (GtkButton              *button,
   /* Select the previous row before removing the current one */
   index = gtk_list_box_row_get_index (list_box_row);
   if (index == 0)
-    index = 2; // Trick in order to select the futur first row
+    index = 2; /* Trick in order to select the futur first row */
   prev_list_box_row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (dialog->search_engine_list_box),
                                                      index - 1);
   /* If the removed engine is the default, choose the newly selected as default */
   default_search_engine = ephy_search_engine_manager_get_default_engine (dialog->search_engine_manager);
 
-  if (g_strcmp0(default_search_engine, search_engine_name) == 0) {
+  if (g_strcmp0 (default_search_engine, search_engine_name) == 0) {
     search_engine_label = container_get_first_child (GTK_CONTAINER (prev_list_box_row));
     search_engine_name = gtk_label_get_text (GTK_LABEL (search_engine_label));
     ephy_search_engine_manager_set_default_engine (dialog->search_engine_manager,
@@ -401,7 +398,7 @@ on_default_search_engine_switch_state_modified (GtkWidget              *switch_w
   search_engine_name = gtk_label_get_text (GTK_LABEL (search_engine_label));
   default_search_engine = ephy_search_engine_manager_get_default_engine (dialog->search_engine_manager);
 
-  if ( g_strcmp0(default_search_engine, search_engine_name) != 0) {
+  if (g_strcmp0 (default_search_engine, search_engine_name) != 0) {
     is_set = ephy_search_engine_manager_set_default_engine (dialog->search_engine_manager,
                                                             search_engine_name);
 
@@ -415,8 +412,7 @@ on_default_search_engine_switch_state_modified (GtkWidget              *switch_w
                            TRUE);
     gtk_widget_set_sensitive (dialog->search_engine_default_switch, FALSE);
     return TRUE;
-  }
-  else
+  } else
     return FALSE;
 }
 
@@ -428,12 +424,6 @@ ephy_search_engine_dialog_class_init (EphySearchEngineDialogClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/epiphany/gtk/search-engine-dialog.ui");
 
-  gtk_widget_class_bind_template_child (widget_class,
-                                        EphySearchEngineDialog,
-                                        search_engine_add_button);
-  gtk_widget_class_bind_template_child (widget_class,
-                                        EphySearchEngineDialog,
-                                        search_engine_remove_button);
   gtk_widget_class_bind_template_child (widget_class,
                                         EphySearchEngineDialog,
                                         search_engine_list_box);
@@ -474,7 +464,7 @@ sort_list_box_by_engine_name (GtkListBoxRow *row1,
 }
 
 static void
-dialog_entry_change_on_event (GtkWidget *entry,
+dialog_entry_change_on_event (GtkWidget              *entry,
                               EphySearchEngineDialog *dialog)
 {
   EphySearchEngineManager *manager;
@@ -535,8 +525,8 @@ dialog_entry_change_on_event (GtkWidget *entry,
 
 static gboolean
 address_entry_on_focus_out_cb (GtkWidget *entry,
-                              GdkEvent  *event,
-                              gpointer   data)
+                               GdkEvent  *event,
+                               gpointer   data)
 {
   EphySearchEngineDialog *dialog;
   dialog = EPHY_SEARCH_ENGINE_DIALOG (data);
@@ -546,7 +536,7 @@ address_entry_on_focus_out_cb (GtkWidget *entry,
 
 static void
 address_entry_on_activate_cb (GtkWidget *entry,
-                             gpointer   data)
+                              gpointer   data)
 {
   EphySearchEngineDialog *dialog;
   dialog = EPHY_SEARCH_ENGINE_DIALOG (data);
@@ -595,7 +585,7 @@ name_entry_change_on_event (GtkWidget              *entry,
   search_engine_label = container_get_first_child (GTK_CONTAINER (list_box_row));
   search_engine_name = gtk_label_get_text (GTK_LABEL (search_engine_label));
   new_search_engine_name = gtk_entry_get_text (GTK_ENTRY (entry));
-  search_engine_address =  gtk_entry_get_text (GTK_ENTRY (dialog->search_engine_address_entry));
+  search_engine_address = gtk_entry_get_text (GTK_ENTRY (dialog->search_engine_address_entry));
   search_engine_bang = gtk_entry_get_text (GTK_ENTRY (dialog->search_engine_bang_entry));
 
   if (g_strcmp0 (search_engine_name, new_search_engine_name) == 0) {
@@ -624,7 +614,7 @@ name_entry_change_on_event (GtkWidget              *entry,
   /* Change the name of the default searche engine too*/
   default_search_engine = ephy_search_engine_manager_get_default_engine (dialog->search_engine_manager);
 
-  if (g_strcmp0(default_search_engine, search_engine_name) == 0)
+  if (g_strcmp0 (default_search_engine, search_engine_name) == 0)
     ephy_search_engine_manager_set_default_engine (dialog->search_engine_manager,
                                                    new_search_engine_name);
 
