@@ -30,6 +30,7 @@
 #include "ephy-gui.h"
 #include "ephy-header-bar.h"
 #include "ephy-history-dialog.h"
+#include "ephy-link.h"
 #include "ephy-lockdown.h"
 #include "ephy-notification.h"
 #include "ephy-prefs.h"
@@ -52,7 +53,6 @@ struct _EphyShell {
 
   EphySession *session;
   EphySyncService *sync_service;
-  GList *windows;
   GObject *lockdown;
   EphyBookmarksManager *bookmarks_manager;
   EphyHistoryManager *history_manager;
@@ -63,6 +63,8 @@ struct _EphyShell {
   EphyShellStartupContext *local_startup_context;
   EphyShellStartupContext *remote_startup_context;
   GSList *open_uris_idle_ids;
+
+  gchar *open_notification_id;
 };
 
 static EphyShell *ephy_shell = NULL;
@@ -115,16 +117,43 @@ ephy_shell_startup_continue (EphyShell               *shell,
                              EphyShellStartupContext *ctx)
 {
   EphySession *session = ephy_shell_get_session (shell);
+  gboolean new_window_option = (ctx->startup_mode == EPHY_STARTUP_NEW_WINDOW);
+  GtkWindow *active_window = gtk_application_get_active_window (GTK_APPLICATION (shell));
 
   if (ctx->session_filename != NULL) {
     g_assert (session != NULL);
     ephy_session_load (session, (const char *)ctx->session_filename,
                        ctx->user_time, NULL, NULL, NULL);
+  } else if (new_window_option) {
+    char *homepage_url = g_settings_get_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_HOMEPAGE_URL);
+    const char *default_uris[] = { homepage_url, NULL };
+    const char **uris = NULL;
+
+    if (ctx->arguments)
+      uris = (const char **)ctx->arguments;
+    else
+      /* If there are no URLs passed in arguments but the --new-window option */
+      /* has been used, then use the default_uris list to open the homepage */
+      uris = default_uris;
+
+    ephy_shell_open_uris (shell, uris, ctx->startup_mode, ctx->user_time);
+    g_free (homepage_url);
+  } else if (active_window && !ctx->arguments) {
+    /* If the application already has an active window and the --new-window */
+    /* option was not passed, then we should just present it */
+    /* This can happen for example if the user starts a long download and then */
+    /* closes the browser.*/
+    /* The window will still remain active and presenting it will have a */
+    /* session-resumed feel for the user. */
+    gtk_window_present (active_window);
   } else if (ctx->arguments || !session) {
     /* Don't queue any window openings if no extra arguments given, */
     /* since session autoresume will open one for us. */
     ephy_shell_open_uris (shell, (const char **)ctx->arguments,
                           ctx->startup_mode, ctx->user_time);
+  } else if (ephy_shell_get_n_windows (shell) == 0) {
+    EphyWindow *window = ephy_window_new ();
+    ephy_link_open (EPHY_LINK (window), NULL, NULL, EPHY_LINK_HOME_PAGE);
   }
 }
 
@@ -244,6 +273,22 @@ quit_application (GSimpleAction *action,
   window_cmd_quit (NULL, NULL, NULL);
 }
 
+static void
+show_downloads (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  GtkWindow *window;
+  EphyDownloadsManager *manager = ephy_embed_shell_get_downloads_manager (EPHY_EMBED_SHELL (ephy_shell_get_default ()));
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+  g_application_withdraw_notification (G_APPLICATION (ephy_shell), ephy_shell->open_notification_id);
+  g_clear_pointer (&ephy_shell->open_notification_id, g_free);
+
+  gtk_widget_show (GTK_WIDGET (window));
+  g_signal_emit_by_name (manager, "show-downloads", NULL);
+}
+
 static GActionEntry app_entries[] = {
   { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
@@ -255,6 +300,7 @@ static GActionEntry app_entries[] = {
   { "help", show_help, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
+  { "show-downloads", show_downloads, NULL, NULL, NULL },
 };
 
 static GActionEntry non_incognito_extra_app_entries[] = {
@@ -412,10 +458,13 @@ create_web_view_for_automation_cb (WebKitAutomationSession *session,
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   n_embeds = ephy_embed_container_get_n_children (EPHY_EMBED_CONTAINER (window));
   web_view = ephy_embed_get_web_view (embed);
-  if (n_embeds == 1 && ephy_web_view_get_visit_type (web_view) == EPHY_PAGE_VISIT_HOMEPAGE)
+  if (n_embeds == 1 && ephy_web_view_get_visit_type (web_view) == EPHY_PAGE_VISIT_HOMEPAGE) {
+    gtk_widget_grab_focus (GTK_WIDGET (embed));
     return GTK_WIDGET (web_view);
+  }
 
   embed = ephy_shell_new_tab (shell, window, NULL, EPHY_NEW_TAB_JUMP);
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
   return GTK_WIDGET (ephy_embed_get_web_view (embed));
 }
 
@@ -457,6 +506,11 @@ ephy_shell_activate (GApplication *application)
     g_signal_connect (web_context, "automation-started", G_CALLBACK (automation_started_cb), shell);
   }
 
+  if (shell->open_notification_id) {
+    g_application_withdraw_notification (G_APPLICATION (shell), shell->open_notification_id);
+    g_clear_pointer (&shell->open_notification_id, g_free);
+  }
+
   if (shell->remote_startup_context == NULL) {
     EphySession *session = ephy_shell_get_session (shell);
 
@@ -501,8 +555,6 @@ ephy_shell_add_platform_data (GApplication    *application,
   EphyShell *app;
   EphyShellStartupContext *ctx;
   GVariantBuilder *ctx_builder;
-  static const char *empty_arguments[] = { "", NULL };
-  const char * const *arguments;
 
   app = EPHY_SHELL (application);
 
@@ -532,13 +584,9 @@ ephy_shell_add_platform_data (GApplication    *application,
      * the primary instance opens a new window.
      */
     if (ctx->arguments)
-      arguments = (const gchar * const *)ctx->arguments;
-    else
-      arguments = empty_arguments;
-
-    g_variant_builder_add (ctx_builder, "{iv}",
-                           CTX_ARGUMENTS,
-                           g_variant_new_strv (arguments, -1));
+      g_variant_builder_add (ctx_builder, "{iv}",
+                             CTX_ARGUMENTS,
+                             g_variant_new_strv ((const char **)ctx->arguments, -1));
 
     g_variant_builder_add (ctx_builder, "{iv}",
                            CTX_USER_TIME,
@@ -593,6 +641,7 @@ ephy_shell_before_emit (GApplication *application,
             break;
         }
       }
+      break;
     }
   }
 
@@ -689,6 +738,11 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
 
+  if (shell->open_notification_id) {
+    g_application_withdraw_notification (G_APPLICATION (shell), shell->open_notification_id);
+    g_clear_pointer (&shell->open_notification_id, g_free);
+  }
+
   g_slist_free_full (shell->open_uris_idle_ids, remove_open_uris_idle_cb);
   shell->open_uris_idle_ids = NULL;
 
@@ -761,12 +815,9 @@ ephy_shell_new_tab_full (EphyShell       *shell,
        window, previous_embed, jump_to ? "t" : "f");
 
   if (flags & EPHY_NEW_TAB_APPEND_AFTER) {
-    if (previous_embed) {
-      GtkWidget *nb = ephy_window_get_notebook (window);
-      /* FIXME this assumes the tab is the  direct notebook child */
-      position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-                                        GTK_WIDGET (previous_embed)) + 1;
-    } else
+    if (previous_embed)
+      position = ephy_window_get_position_for_new_embed (window, previous_embed);
+    else
       g_warning ("Requested to append new tab after parent, but 'previous_embed' was NULL");
   }
 
@@ -832,7 +883,7 @@ ephy_shell_get_session (EphyShell *shell)
   g_assert (EPHY_IS_SHELL (shell));
 
   mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell));
-  if (mode ==  EPHY_EMBED_SHELL_MODE_APPLICATION || mode == EPHY_EMBED_SHELL_MODE_INCOGNITO || mode == EPHY_EMBED_SHELL_MODE_AUTOMATION)
+  if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION || mode == EPHY_EMBED_SHELL_MODE_INCOGNITO || mode == EPHY_EMBED_SHELL_MODE_AUTOMATION)
     return NULL;
 
   if (shell->session == NULL)
@@ -988,16 +1039,19 @@ ephy_shell_get_prefs_dialog (EphyShell *shell)
 void
 _ephy_shell_create_instance (EphyEmbedShellMode mode)
 {
-  const char *id;
+  g_autofree gchar *id = NULL;
 
   g_assert (ephy_shell == NULL);
 
-  id = (mode == EPHY_EMBED_SHELL_MODE_APPLICATION ? "org.gnome.Epiphany.WebApp"
-                                                  : "org.gnome.Epiphany");
+  if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION)
+    id = g_strconcat (APPLICATION_ID, ".WebApp", NULL);
+  else
+    id = g_strdup (APPLICATION_ID);
 
   ephy_shell = EPHY_SHELL (g_object_new (EPHY_TYPE_SHELL,
                                          "application-id", id,
                                          "mode", mode,
+                                         "resource-base-path", "/org/gnome/Epiphany",
                                          NULL));
   /* FIXME weak ref */
   g_assert (ephy_shell != NULL);
@@ -1057,6 +1111,11 @@ ephy_shell_close_all_windows (EphyShell *shell)
       retval = FALSE;
   }
 
+  if (shell->open_notification_id) {
+    g_application_withdraw_notification (G_APPLICATION (shell), shell->open_notification_id);
+    g_clear_pointer (&shell->open_notification_id, g_free);
+  }
+
   return retval;
 }
 
@@ -1209,3 +1268,17 @@ ephy_shell_open_uris (EphyShell        *shell,
 
   shell->open_uris_idle_ids = g_slist_prepend (shell->open_uris_idle_ids, GUINT_TO_POINTER (id));
 }
+
+void
+ephy_shell_send_notification (EphyShell     *shell,
+                              gchar         *id,
+                              GNotification *notification)
+{
+  if (ephy_shell->open_notification_id) {
+    g_application_withdraw_notification (G_APPLICATION (ephy_shell), ephy_shell->open_notification_id);
+    g_clear_pointer (&ephy_shell->open_notification_id, g_free);
+  }
+
+  shell->open_notification_id = g_strdup (id);
+  g_application_send_notification (G_APPLICATION (shell), id, notification);
+}
