@@ -22,6 +22,7 @@
 #include "ephy-embed-shell.h"
 #include "ephy-search-engine-manager.h"
 #include "ephy-suggestion.h"
+#include "ephy-window.h"
 
 #include <dazzle.h>
 #include <glib/gi18n.h>
@@ -29,12 +30,12 @@
 #define MAX_COMPLETION_HISTORY_URLS 8
 
 struct _EphySuggestionModel {
-  GObject               parent;
-  EphyHistoryService   *history_service;
+  GObject parent;
+  EphyHistoryService *history_service;
   EphyBookmarksManager *bookmarks_manager;
-  GSequence            *items;
-  gchar               **search_terms;
-  GCancellable         *icon_cancellable;
+  GSequence *urls;
+  GSequence *items;
+  GCancellable *icon_cancellable;
 };
 
 enum {
@@ -58,13 +59,12 @@ ephy_suggestion_model_finalize (GObject *object)
 
   g_clear_object (&self->bookmarks_manager);
   g_clear_object (&self->history_service);
+  g_clear_pointer (&self->urls, g_sequence_free);
   g_clear_pointer (&self->items, g_sequence_free);
 
   g_cancellable_cancel (self->icon_cancellable);
   g_clear_object (&self->icon_cancellable);
 
-  g_strfreev (self->search_terms);
-
   G_OBJECT_CLASS (ephy_suggestion_model_parent_class)->finalize (object);
 }
 
@@ -189,40 +189,17 @@ ephy_suggestion_model_new (EphyHistoryService   *history_service,
                        NULL);
 }
 
-static void
-update_search_terms (EphySuggestionModel *self,
-                     const char          *text)
-{
-  g_assert (EPHY_IS_SUGGESTION_MODEL (self));
-
-  g_strfreev (self->search_terms);
-
-  self->search_terms = g_strsplit (text, " ", -1);
-}
-
 static gboolean
 should_add_bookmark_to_model (EphySuggestionModel *self,
                               const char          *search_string,
                               const char          *title,
                               const char          *location)
 {
-  gboolean ret = TRUE;
+  g_autofree gchar *search_casefold = g_utf8_casefold (search_string, -1);
+  g_autofree gchar *title_casefold = g_utf8_casefold (title, -1);
+  g_autofree gchar *location_casefold = g_utf8_casefold (location, -1);
 
-  if (self->search_terms) {
-    guint len = g_strv_length (self->search_terms);
-    guint i;
-
-    for (i = 0; i < len; i++) {
-      gchar *str = self->search_terms[i];
-
-      if (!strstr (str, title ? title : "") && !strstr (str, location ? location : "")) {
-        ret = FALSE;
-        break;
-      }
-    }
-  }
-
-  return ret;
+  return strstr (title_casefold, search_casefold) || strstr (location_casefold, search_casefold);
 }
 
 static void
@@ -274,6 +251,7 @@ static guint
 add_bookmarks (EphySuggestionModel *self,
                const char          *query)
 {
+  g_autoptr (GList) new_urls = NULL;
   GSequence *bookmarks;
   guint added = 0;
 
@@ -290,6 +268,10 @@ add_bookmarks (EphySuggestionModel *self,
     url = ephy_bookmark_get_url (bookmark);
     title = ephy_bookmark_get_title (bookmark);
 
+    if (g_sequence_lookup (self->urls, (gpointer)url, (GCompareDataFunc)g_strcmp0,
+                           NULL))
+      continue;
+
     if (strlen (title) == 0)
       title = url;
 
@@ -300,14 +282,20 @@ add_bookmarks (EphySuggestionModel *self,
 
       escaped_title = g_markup_escape_text (title, -1);
       markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-      suggestion = ephy_suggestion_new (markup, url);
+      suggestion = ephy_suggestion_new (markup, title, url);
       load_favicon (self, suggestion, url);
+      ephy_suggestion_set_secondary_icon (suggestion, "starred-symbolic");
 
+      new_urls = g_list_prepend (new_urls, g_strdup (url));
       g_sequence_append (self->items, suggestion);
       added++;
     }
   }
 
+  for (GList *p = new_urls; p != NULL; p = p->next)
+    g_sequence_append (self->urls, g_steal_pointer (&p->data));
+  g_sequence_sort (self->urls, (GCompareDataFunc)g_strcmp0, NULL);
+
   return added;
 }
 
@@ -325,13 +313,17 @@ add_history (EphySuggestionModel *self,
     g_autofree gchar *markup = NULL;
     const gchar *title = url->title;
 
+    if (g_sequence_lookup (self->urls, url->url, (GCompareDataFunc)g_strcmp0,
+                           NULL))
+      continue;
+
     if (strlen (url->title) == 0)
       title = url->url;
 
     escaped_title = g_markup_escape_text (title, -1);
 
     markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-    suggestion = ephy_suggestion_new (markup, url->url);
+    suggestion = ephy_suggestion_new (markup, title, url->url);
     load_favicon (self, suggestion, url->url);
 
     g_sequence_append (self->items, suggestion);
@@ -356,20 +348,26 @@ add_search_engines (EphySuggestionModel *self,
 
   for (guint i = 0; engines[i] != NULL; i++) {
     EphySuggestion *suggestion;
-    char *address;
-    g_autofree gchar *escaped_title = NULL;
-    g_autofree gchar *markup = NULL;
+    g_autofree char *address = NULL;
+    g_autofree char *escaped_title = NULL;
+    g_autofree char *markup = NULL;
+    g_autoptr (SoupURI) uri = NULL;
 
     address = ephy_search_engine_manager_build_search_address (manager, engines[i], query);
     escaped_title = g_markup_escape_text (engines[i], -1);
     markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-    suggestion = ephy_suggestion_new_without_subtitle (markup, address);
+    suggestion = ephy_suggestion_new (markup, engines[i], address);
+
+    uri = soup_uri_new (address);
+    if (uri) {
+      g_free (address);
+      address = g_strdup_printf ("%s://%s/", soup_uri_get_scheme (uri), soup_uri_get_host (uri));
+    }
+
     load_favicon (self, suggestion, address);
 
     g_sequence_append (self->items, suggestion);
     added++;
-
-    g_free (address);
   }
 
   g_strfreev (engines);
@@ -377,6 +375,80 @@ add_search_engines (EphySuggestionModel *self,
   return added;
 }
 
+static guint
+add_tabs (EphySuggestionModel *self,
+          const char          *query)
+{
+  GApplication *application;
+  EphyEmbedShell *shell;
+  EphyWindow *window;
+  GtkWidget *notebook;
+  gint n_pages;
+  gint current;
+  guint added = 0;
+
+  shell = ephy_embed_shell_get_default ();
+  application = G_APPLICATION (shell);
+  window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (application)));
+
+  if (!window)
+    return 0;
+
+  notebook = ephy_window_get_notebook (window);
+  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+  current = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+
+  for (int i = 0; i < n_pages; i++) {
+    EphyEmbed *embed;
+    EphyWebView *webview;
+    EphySuggestion *suggestion;
+    g_autofree gchar *escaped_title = NULL;
+    g_autofree gchar *markup = NULL;
+    const gchar *display_address;
+    const gchar *url;
+    g_autofree gchar *address = NULL;
+    const gchar *title;
+    g_autofree gchar *title_casefold = NULL;
+    g_autofree gchar *display_address_casefold = NULL;
+    g_autofree gchar *query_casefold = NULL;
+
+    if (i == current)
+      continue;
+
+    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+    webview = ephy_embed_get_web_view (embed);
+    display_address = ephy_web_view_get_display_address (webview);
+    url = ephy_web_view_get_address (webview);
+    address = g_strdup_printf ("ephy-tab://%d", i);
+    title = webkit_web_view_get_title (WEBKIT_WEB_VIEW (webview));
+
+    display_address_casefold = g_utf8_casefold (display_address, -1);
+    query_casefold = g_utf8_casefold (query, -1);
+    if (!title)
+      title = "";
+
+    title_casefold = g_utf8_casefold (title, -1);
+
+    if ((title_casefold && strstr (title_casefold, query_casefold)) || strstr (display_address_casefold, query_casefold)) {
+      char *escaped_address = g_markup_escape_text (display_address, -1);
+
+      escaped_title = g_markup_escape_text (title, -1);
+      markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
+      suggestion = ephy_suggestion_new_with_custom_subtitle (markup, title, escaped_address, address);
+      load_favicon (self, suggestion, display_address);
+      ephy_suggestion_set_secondary_icon (suggestion, "go-jump-symbolic");
+
+      g_sequence_append (self->urls, g_strdup (url));
+      g_sequence_append (self->items, suggestion);
+      added++;
+    }
+  }
+
+  g_sequence_sort (self->urls, (GCompareDataFunc)g_strcmp0, NULL);
+
+  return added;
+}
+
 static void
 query_completed_cb (EphyHistoryService *service,
                     gboolean            success,
@@ -401,11 +473,14 @@ query_completed_cb (EphyHistoryService *service,
 
   removed = g_sequence_get_length (self->items);
 
+  g_clear_pointer (&self->urls, g_sequence_free);
+  self->urls = g_sequence_new (g_free);
   g_clear_pointer (&self->items, g_sequence_free);
   self->items = g_sequence_new (g_object_unref);
 
   if (strlen (query) > 0) {
-    added = add_bookmarks (self, query);
+    added = add_tabs (self, query);
+    added += add_bookmarks (self, query);
     added += add_history (self, urls, query);
     added += add_search_engines (self, query);
   }
@@ -440,8 +515,6 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
   for (guint i = 0; strings[i]; i++)
     qlist = g_list_append (qlist, g_strdup (strings[i]));
 
-  update_search_terms (self, query);
-
   ephy_history_service_find_urls (self->history_service,
                                   0, 0,
                                   MAX_COMPLETION_HISTORY_URLS, 0,
@@ -470,6 +543,7 @@ ephy_suggestion_model_get_suggestion_with_uri (EphySuggestionModel *self,
                                                const char          *uri)
 {
   GSequenceIter *iter;
+  g_autofree gchar *uri_casefold = g_utf8_casefold (uri, -1);
 
   g_assert (EPHY_IS_SUGGESTION_MODEL (self));
   g_assert (uri != NULL && *uri != '\0');
@@ -477,9 +551,12 @@ ephy_suggestion_model_get_suggestion_with_uri (EphySuggestionModel *self,
   for (iter = g_sequence_get_begin_iter (self->items);
        !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
     EphySuggestion *suggestion;
+    g_autofree gchar *suggestion_casefold = NULL;
 
     suggestion = g_sequence_get (iter);
-    if (strcmp (ephy_suggestion_get_uri (suggestion), uri) == 0)
+    suggestion_casefold = g_utf8_casefold (ephy_suggestion_get_uri (suggestion), -1);
+
+    if (strcmp (suggestion_casefold, uri_casefold) == 0)
       return suggestion;
   }
 
