@@ -52,7 +52,6 @@
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-string.h"
-#include "ephy-vcs-version.h"
 #include "ephy-view-source-handler.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-zoom.h"
@@ -218,7 +217,8 @@ create_tree_model (void)
 }
 
 static gchar *
-show_profile_selector (GtkWidget *parent, GSList *profiles)
+show_profile_selector (GtkWidget *parent,
+                       GSList    *profiles)
 {
   GtkWidget *selector;
   GtkWidget *list_box;
@@ -319,8 +319,8 @@ dialog_bookmarks_import_cb (GtkDialog   *dialog,
                                                      imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
                                                      GTK_BUTTONS_OK,
                                                      "%s",
-                                                     imported ? _("Bookmarks successfully imported!") :
-                                                                error->message);
+                                                     imported ? _("Bookmarks successfully imported!")
+                                                              : error->message);
         gtk_dialog_run (GTK_DIALOG (import_info_dialog));
 
         gtk_widget_destroy (import_info_dialog);
@@ -354,14 +354,14 @@ dialog_bookmarks_import_cb (GtkDialog   *dialog,
        * the profile (he pressed Cancel), don't display the import info dialog
        * as no import took place
        */
-      if (profile) {
+      if (num_profiles == 1 || profile) {
         import_info_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
                                                      GTK_DIALOG_MODAL,
                                                      imported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
                                                      GTK_BUTTONS_OK,
                                                      "%s",
-                                                     imported ? _("Bookmarks successfully imported!") :
-                                                                error->message);
+                                                     imported ? _("Bookmarks successfully imported!")
+                                                              : error->message);
         gtk_dialog_run (GTK_DIALOG (import_info_dialog));
         gtk_widget_destroy (import_info_dialog);
       }
@@ -436,6 +436,33 @@ window_cmd_import_bookmarks (GSimpleAction *action,
   gtk_widget_show_all (dialog);
 }
 
+static void
+bookmarks_export_cb (GObject      *source_object,
+                     GAsyncResult *result,
+                     gpointer      user_data)
+{
+  EphyBookmarksManager *manager = EPHY_BOOKMARKS_MANAGER (source_object);
+  GtkWidget *window = GTK_WIDGET (user_data);
+  GtkWidget *export_info_dialog;
+  gboolean exported;
+  g_autoptr (GError) error = NULL;
+
+  exported = ephy_bookmarks_export_finish (manager, result, &error);
+
+  export_info_dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                                               GTK_DIALOG_MODAL,
+                                               exported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
+                                               GTK_BUTTONS_OK,
+                                               "%s",
+                                               exported ? _("Bookmarks successfully exported!") :
+                                               error->message);
+  gtk_dialog_run (GTK_DIALOG (export_info_dialog));
+  gtk_widget_destroy (export_info_dialog);
+
+  g_object_unref (manager);
+  g_object_unref (window);
+}
+
 void
 window_cmd_export_bookmarks (GSimpleAction *action,
                              GVariant      *parameter,
@@ -443,13 +470,12 @@ window_cmd_export_bookmarks (GSimpleAction *action,
 {
   EphyBookmarksManager *manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
   GtkFileChooser *dialog;
-  GtkWidget *export_info_dialog;
   int chooser_response;
-  gboolean exported;
   GtkFileFilter *filter;
+  GtkWindow *window = user_data;
 
   dialog = GTK_FILE_CHOOSER (gtk_file_chooser_native_new (_("Choose File"),
-                                                          GTK_WINDOW (user_data),
+                                                          window,
                                                           GTK_FILE_CHOOSER_ACTION_SAVE,
                                                           _("_Save"),
                                                           _("_Cancel")));
@@ -464,24 +490,17 @@ window_cmd_export_bookmarks (GSimpleAction *action,
 
   chooser_response = gtk_native_dialog_run (GTK_NATIVE_DIALOG (dialog));
   if (chooser_response == GTK_RESPONSE_ACCEPT) {
-    GError *error = NULL;
     char *filename;
 
     gtk_native_dialog_hide (GTK_NATIVE_DIALOG (dialog));
 
     filename = gtk_file_chooser_get_filename (dialog);
-    exported = ephy_bookmarks_export (manager, filename, &error);
+    ephy_bookmarks_export (g_object_ref (manager),
+                           filename,
+                           NULL,
+                           bookmarks_export_cb,
+                           g_object_ref (window));
     g_free (filename);
-
-    export_info_dialog = gtk_message_dialog_new (GTK_WINDOW (user_data),
-                                                 GTK_DIALOG_MODAL,
-                                                 exported ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
-                                                 GTK_BUTTONS_OK,
-                                                 "%s",
-                                                 exported ? _("Bookmarks successfully exported!") :
-                                                            error->message);
-    gtk_dialog_run (GTK_DIALOG (export_info_dialog));
-    gtk_widget_destroy (export_info_dialog);
   }
 
   g_object_unref (dialog);
@@ -566,7 +585,11 @@ window_cmd_show_about (GSimpleAction *action,
   GKeyFile *key_file;
   GBytes *bytes;
   GError *error = NULL;
-  char **authors, **maintainers, **past_maintainers, **contributors, **artists, **documenters;
+  char **orig_authors, **maintainers, **past_maintainers, **contributors, **artists, **documenters;
+  char **authors = NULL;
+  guint index;
+  guint author_index = 0;
+  guint length;
 
   key_file = g_key_file_new ();
   bytes = g_resources_lookup_data ("/org/gnome/epiphany/about.ini", 0, NULL);
@@ -578,7 +601,7 @@ window_cmd_show_about (GSimpleAction *action,
   }
   g_bytes_unref (bytes);
 
-  authors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Author", NULL, NULL);
+  orig_authors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Author", NULL, NULL);
   maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Maintainers", NULL, NULL);
   past_maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "PastMaintainers", NULL, NULL);
   contributors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Contributors", NULL, NULL);
@@ -587,7 +610,7 @@ window_cmd_show_about (GSimpleAction *action,
   g_key_file_free (key_file);
 
   comments = g_strdup_printf (_("A simple, clean, beautiful view of the web.\n"
-                                "Powered by WebKitGTK+ %d.%d.%d"),
+                                "Powered by WebKitGTK %d.%d.%d"),
                               webkit_get_major_version (),
                               webkit_get_minor_version (),
                               webkit_get_micro_version ());
@@ -603,24 +626,47 @@ window_cmd_show_about (GSimpleAction *action,
   gtk_about_dialog_set_program_name (dialog, _("Epiphany Technology Preview"));
 #endif
 
-  gtk_about_dialog_set_version (dialog, VCSVERSION);
-  gtk_about_dialog_set_copyright (dialog, "Copyright © 2002–2004 Marco Pesenti Gritti\n"
-                                          "Copyright © 2003–2019 The GNOME Web Developers");
+  gtk_about_dialog_set_version (dialog, VERSION);
+  gtk_about_dialog_set_copyright (dialog,
+                                  "Copyright © 2002–2004 Marco Pesenti Gritti\n"
+                                  "Copyright © 2003–2020 The GNOME Web Developers");
   gtk_about_dialog_set_comments (dialog, comments);
   gtk_about_dialog_set_license_type (dialog, GTK_LICENSE_GPL_3_0);
   gtk_about_dialog_set_website (dialog, "https://wiki.gnome.org/Apps/Web");
   gtk_about_dialog_set_website_label (dialog, _("Website"));
-  gtk_about_dialog_set_logo_icon_name (dialog, "org.gnome.Epiphany");
+  gtk_about_dialog_set_logo_icon_name (dialog, APPLICATION_ID);
+
+  length = g_strv_length (orig_authors) +
+           g_strv_length (maintainers) +
+           g_strv_length (past_maintainers) +
+           g_strv_length (contributors) +
+           3;
+
+  authors = g_malloc0 (sizeof (char *) * (length + 1));
+
+  for (index = 0; index < g_strv_length (orig_authors); index++)
+    authors[author_index++] = g_strdup (orig_authors[index]);
+
+  authors[author_index++] = g_strdup ("");
+
+  for (index = 0; index < g_strv_length (maintainers); index++)
+    authors[author_index++] = g_strdup (maintainers[index]);
+
+  authors[author_index++] = g_strdup ("");
+
+  for (index = 0; index < g_strv_length (past_maintainers); index++)
+    authors[author_index++] = g_strdup (past_maintainers[index]);
+
+  authors[author_index++] = g_strdup ("");
+
+  for (index = 0; index < g_strv_length (contributors); index++) {
+    authors[author_index++] = g_strdup (contributors[index]);
+  }
 
-  /* We have to use add_credit_section() for documenters and artists to
-   * ensure maintainers get sorted first. */
   gtk_about_dialog_set_authors (dialog, (const char **)authors);
+  gtk_about_dialog_set_artists (dialog, (const char **)artists);
+  gtk_about_dialog_set_documenters (dialog, (const char **)documenters);
   gtk_about_dialog_set_translator_credits (dialog, _("translator-credits"));
-  gtk_about_dialog_add_credit_section (dialog, _("Current maintainers"), (const char **)maintainers);
-  gtk_about_dialog_add_credit_section (dialog, _("Past maintainers"), (const char **)past_maintainers);
-  gtk_about_dialog_add_credit_section (dialog, _("Documented by"), (const char **)documenters);
-  gtk_about_dialog_add_credit_section (dialog, _("Artwork by"), (const char **)artists);
-  gtk_about_dialog_add_credit_section (dialog, _("Contributors"), (const char **)contributors);
 
   gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -661,6 +707,12 @@ window_cmd_navigation (GSimpleAction *action,
   EphyWindow *window = EPHY_WINDOW (user_data);
   EphyEmbed *embed;
   WebKitWebView *web_view;
+  EphyLinkFlags flags = ephy_link_flags_from_current_event ();
+
+  if (flags & EPHY_LINK_NEW_TAB) {
+    window_cmd_navigation_new_tab (action, parameter, user_data);
+    return;
+  }
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
@@ -691,41 +743,41 @@ window_cmd_navigation_new_tab (GSimpleAction *action,
   web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
   if (strstr (g_action_get_name (G_ACTION (action)), "back")) {
-      const char *back_uri;
-      WebKitBackForwardList *history;
-      WebKitBackForwardListItem *back_item;
+    const char *back_uri;
+    WebKitBackForwardList *history;
+    WebKitBackForwardListItem *back_item;
 
-      history = webkit_web_view_get_back_forward_list (web_view);
-      back_item = webkit_back_forward_list_get_back_item (history);
-      back_uri = webkit_back_forward_list_item_get_original_uri (back_item);
+    history = webkit_web_view_get_back_forward_list (web_view);
+    back_item = webkit_back_forward_list_get_back_item (history);
+    back_uri = webkit_back_forward_list_item_get_original_uri (back_item);
 
-      embed = ephy_shell_new_tab (ephy_shell_get_default (),
-                                  EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
-                                  NULL,
-                                  0);
+    embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                                EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
+                                NULL,
+                                0);
 
-      web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
-      webkit_web_view_load_uri (web_view, back_uri);
-      gtk_widget_grab_focus (GTK_WIDGET (embed));
+    web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+    webkit_web_view_load_uri (web_view, back_uri);
+    gtk_widget_grab_focus (GTK_WIDGET (embed));
   } else {
-      const char *forward_uri;
-      WebKitBackForwardList *history;
-      WebKitBackForwardListItem *forward_item;
+    const char *forward_uri;
+    WebKitBackForwardList *history;
+    WebKitBackForwardListItem *forward_item;
 
-      /* Forward history is not copied when opening
-         a new tab, so get the forward URI manually
-         and load it */
-      history = webkit_web_view_get_back_forward_list (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
-      forward_item = webkit_back_forward_list_get_forward_item (history);
-      forward_uri = webkit_back_forward_list_item_get_original_uri (forward_item);
+    /* Forward history is not copied when opening
+     *  a new tab, so get the forward URI manually
+     *  and load it */
+    history = webkit_web_view_get_back_forward_list (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
+    forward_item = webkit_back_forward_list_get_forward_item (history);
+    forward_uri = webkit_back_forward_list_item_get_original_uri (forward_item);
 
-      embed = ephy_shell_new_tab (ephy_shell_get_default (),
-                                  EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
-                                  embed,
-                                  0);
+    embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                                EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
+                                embed,
+                                0);
 
-      web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
-      webkit_web_view_load_uri (web_view, forward_uri);
+    web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+    webkit_web_view_load_uri (web_view, forward_uri);
   }
 }
 
@@ -826,15 +878,28 @@ window_cmd_new_tab (GSimpleAction *action,
                     GVariant      *parameter,
                     gpointer       user_data)
 {
+  GtkWidget *stack;
   EphyWindow *window = user_data;
+  char *url;
+
+  stack = ephy_window_get_stack (window);
+  url = g_settings_get_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_HOMEPAGE_URL);
+  if (g_strcmp0 (url, "about:blank") != 0) {
+    g_free (url);
+    url = NULL;
+  }
 
   ephy_link_open (EPHY_LINK (window),
-                  NULL, NULL,
+                  url, NULL,
                   EPHY_LINK_NEW_TAB | EPHY_LINK_JUMP_TO);
+  gtk_stack_set_visible_child_name (GTK_STACK (stack), "content");
+  g_free (url);
 }
 
 static void
-open_response_cb (GtkNativeDialog *dialog, int response, EphyWindow *window)
+open_response_cb (GtkNativeDialog *dialog,
+                  int              response,
+                  EphyWindow      *window)
 {
   if (response == GTK_RESPONSE_ACCEPT) {
     char *uri, *converted;
@@ -882,11 +947,14 @@ typedef struct {
   GtkWidget *box;
   char *icon_href;
   GdkRGBA icon_rgba;
+  GCancellable *cancellable;
 } EphyApplicationDialogData;
 
 static void
 ephy_application_dialog_data_free (EphyApplicationDialogData *data)
 {
+  g_cancellable_cancel (data->cancellable);
+  g_object_unref (data->cancellable);
   g_free (data->icon_href);
   g_free (data);
 }
@@ -1107,8 +1175,12 @@ fill_default_application_image_cb (GObject      *source,
   EphyApplicationDialogData *data = user_data;
   char *uri = NULL;
   GdkRGBA color = { 0.5, 0.5, 0.5, 0.3 };
+  GError *error = NULL;
 
-  ephy_web_view_get_best_web_app_icon_finish (EPHY_WEB_VIEW (source), async_result, &uri, &color, NULL);
+  ephy_web_view_get_best_web_app_icon_finish (EPHY_WEB_VIEW (source), async_result, &uri, &color, &error);
+
+  if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+    return;
 
   data->icon_href = uri;
   data->icon_rgba = color;
@@ -1123,7 +1195,7 @@ fill_default_application_image_cb (GObject      *source,
 static void
 fill_default_application_image (EphyApplicationDialogData *data)
 {
-  ephy_web_view_get_best_web_app_icon (data->view, NULL, fill_default_application_image_cb, data);
+  ephy_web_view_get_best_web_app_icon (data->view, data->cancellable, fill_default_application_image_cb, data);
 }
 
 typedef struct {
@@ -1135,7 +1207,6 @@ static SiteInfo sites[] = {
   { "www.facebook.com", "Facebook" },
   { "twitter.com", "Twitter" },
   { "gmail.com", "GMail" },
-  { "plus.google.com", "Google+" },
   { "youtube.com", "YouTube" },
 };
 
@@ -1195,15 +1266,18 @@ fill_default_application_title_cb (GObject      *source,
 {
   EphyApplicationDialogData *data = user_data;
   char *title;
+  GError *error = NULL;
 
-  title = ephy_web_view_get_web_app_title_finish (EPHY_WEB_VIEW (source), async_result, NULL);
-  set_default_application_title (data, title);
+  /* Confusing: this can return NULL for no title, even when there is no error. */
+  title = ephy_web_view_get_web_app_title_finish (EPHY_WEB_VIEW (source), async_result, &error);
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+    set_default_application_title (data, title);
 }
 
 static void
 fill_default_application_title (EphyApplicationDialogData *data)
 {
-  ephy_web_view_get_web_app_title (data->view, NULL, fill_default_application_title_cb, data);
+  ephy_web_view_get_web_app_title (data->view, data->cancellable, fill_default_application_title_cb, data);
 }
 
 static void
@@ -1222,7 +1296,8 @@ notify_launch_cb (NotifyNotification *notification,
 }
 
 static gboolean
-confirm_web_application_overwrite (GtkWindow *parent, const char *title)
+confirm_web_application_overwrite (GtkWindow  *parent,
+                                   const char *title)
 {
   GtkResponseType response;
   GtkWidget *dialog;
@@ -1255,7 +1330,7 @@ session_bus_ready_cb (GObject      *source,
                       GAsyncResult *res,
                       gpointer      user_data)
 {
-  g_autoptr(GDBusConnection) connection = g_bus_get_finish (res, NULL);
+  g_autoptr (GDBusConnection) connection = g_bus_get_finish (res, NULL);
   g_autofree gchar *desktop_file = user_data;
   g_autofree gchar *app_id = NULL;
   GVariant *app;
@@ -1425,6 +1500,7 @@ window_cmd_save_as_application (GSimpleAction *action,
   data->image = image;
   data->entry = entry;
   data->spinner = spinner;
+  data->cancellable = g_cancellable_new ();
 
   g_object_bind_property (image, "visible", spinner, "visible", G_BINDING_INVERT_BOOLEAN);
 
@@ -1476,8 +1552,79 @@ get_suggested_filename (EphyEmbed *embed)
   return suggested_filename;
 }
 
+
 static void
-save_response_cb (GtkNativeDialog *dialog, int response, EphyEmbed *embed)
+save_snapshot (cairo_surface_t *surface,
+               const char      *file)
+{
+  g_autoptr (GdkPixbuf) pixbuf = NULL;
+  g_autofree char *snapshot_path = NULL;
+  g_autoptr (GError) error = NULL;
+  int width;
+  int height;
+  gboolean ret;
+
+  /* Create a pixbuf */
+  width = cairo_image_surface_get_width (surface);
+  height = cairo_image_surface_get_height (surface);
+
+  pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
+  if (!pixbuf)
+    return;
+
+  ret = gdk_pixbuf_save (pixbuf, file, "png", &error, NULL);
+  if (!ret) {
+    g_warning ("Failed to save image to %s: %s", snapshot_path, error->message);
+    return;
+  }
+}
+
+static void
+take_snapshot_full_cb (GObject      *source,
+                       GAsyncResult *res,
+                       gpointer      user_data)
+{
+  WebKitWebView *view = WEBKIT_WEB_VIEW (source);
+  GError *error = NULL;
+  cairo_surface_t *surface;
+  gchar *file = user_data;
+
+  if (!file)
+    return;
+
+  surface = webkit_web_view_get_snapshot_finish (view, res, &error);
+  if (error) {
+    g_warning ("Failed to take snapshot: %s", error->message);
+    return;
+  }
+
+  save_snapshot (surface, file);
+  cairo_surface_destroy (surface);
+
+  g_free (file);
+  g_object_unref (view);
+}
+
+void
+take_snapshot (EphyEmbed *embed,
+               char      *file)
+{
+  WebKitWebView *view;
+
+  view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+  webkit_web_view_get_snapshot (g_object_ref (view),
+                                WEBKIT_SNAPSHOT_REGION_FULL_DOCUMENT,
+                                WEBKIT_SNAPSHOT_OPTIONS_NONE,
+                                NULL,
+                                take_snapshot_full_cb,
+                                g_filename_from_uri (file, NULL, NULL));
+}
+
+
+static void
+save_response_cb (GtkNativeDialog *dialog,
+                  int              response,
+                  EphyEmbed       *embed)
 {
   if (response == GTK_RESPONSE_ACCEPT) {
     char *uri, *converted;
@@ -1487,8 +1634,12 @@ save_response_cb (GtkNativeDialog *dialog, int response, EphyEmbed *embed)
       converted = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
 
       if (converted != NULL) {
-        EphyWebView *web_view = ephy_embed_get_web_view (embed);
-        ephy_web_view_save (web_view, converted);
+        if (g_str_has_suffix (converted, ".png")) {
+          take_snapshot (embed, converted);
+        } else {
+          EphyWebView *web_view = ephy_embed_get_web_view (embed);
+          ephy_web_view_save (web_view, converted);
+        }
       }
 
       g_free (converted);
@@ -1509,6 +1660,7 @@ window_cmd_save_as (GSimpleAction *action,
   EphyWindow *window = user_data;
   EphyEmbed *embed;
   GtkFileChooser *dialog;
+  GtkFileFilter *filter;
   char *suggested_filename;
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
@@ -1522,6 +1674,21 @@ window_cmd_save_as (GSimpleAction *action,
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
   gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY));
 
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (GTK_FILE_FILTER (filter), _("HTML"));
+  gtk_file_filter_add_pattern (GTK_FILE_FILTER (filter), "*.html");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (GTK_FILE_FILTER (filter), _("MHTML"));
+  gtk_file_filter_add_pattern (GTK_FILE_FILTER (filter), "*.mhtml");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (GTK_FILE_FILTER (filter), _("PNG"));
+  gtk_file_filter_add_pattern (GTK_FILE_FILTER (filter), "*.png");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+
   suggested_filename = ephy_sanitize_filename (get_suggested_filename (embed));
 
   gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), suggested_filename);
@@ -1789,26 +1956,42 @@ window_cmd_encoding (GSimpleAction *action,
   gtk_dialog_run (GTK_DIALOG (dialog));
 }
 
-static void
-view_source_embedded (const char *uri, EphyEmbed *embed)
+void
+window_cmd_page_source (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       user_data)
 {
+  EphyWindow *window = user_data;
+  EphyEmbed *embed;
   EphyEmbed *new_embed;
   SoupURI *soup_uri;
   char *source_uri;
+  const char *address;
+  guint port;
+
+  embed = ephy_embed_container_get_active_child
+            (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
   /* Abort if we're already in view source mode */
-  if (strstr (uri, EPHY_VIEW_SOURCE_SCHEME) == uri)
+  if (strstr (address, EPHY_VIEW_SOURCE_SCHEME) == address)
     return;
 
-  soup_uri = soup_uri_new (uri);
+  soup_uri = soup_uri_new (address);
   if (!soup_uri) {
-    g_critical ("Failed to construct SoupURI for %s", uri);
+    g_critical ("Failed to construct SoupURI for %s", address);
     return;
   }
 
-  /* Convert e.g. https://gnome.org to ephy-source://gnome.org#https */
+  /* Convert e.g. https://gnome.org to ephy-source://gnome.org#https,
+   * taking care to prevent soup_uri_set_scheme() from forcing the default port.
+   */
+  port = soup_uri_get_port (soup_uri);
   soup_uri_set_fragment (soup_uri, soup_uri->scheme);
   soup_uri_set_scheme (soup_uri, EPHY_VIEW_SOURCE_SCHEME);
+  soup_uri_set_port (soup_uri, port);
   source_uri = soup_uri_to_string (soup_uri, FALSE);
 
   new_embed = ephy_shell_new_tab
@@ -1824,277 +2007,6 @@ view_source_embedded (const char *uri, EphyEmbed *embed)
   soup_uri_free (soup_uri);
 }
 
-static void
-tmp_source_delete_cb (GObject *source, GAsyncResult *result, gpointer data)
-{
-  GFile *file = G_FILE (source);
-  GError *error = NULL;
-
-  if (!g_file_delete_finish (G_FILE (source), result, &error)) {
-    g_warning ("Failed to delete %s: %s", g_file_get_path (file), error->message);
-    g_error_free (error);
-  }
-
-  g_object_unref (file);
-}
-
-static void
-delete_tmp_source_file (GOutputStream *ostream)
-{
-  char *uri;
-  GFile *file;
-
-  g_object_get (G_OBJECT (ostream), "ephy-save-temp-source-uri", &uri, NULL);
-  file = g_file_new_for_uri (uri);
-  g_file_delete_async (file, G_PRIORITY_LOW, NULL, tmp_source_delete_cb, NULL);
-
-  g_free (uri);
-}
-
-static void
-portal_opened_cb (GObject *source, GAsyncResult *result, gpointer data)
-{
-  char *path = data;
-  GFile *file;
-  gboolean ret;
-  GError *error = NULL;
-
-  ret = ephy_open_file_via_flatpak_portal_finish (result, &error);
-  if (!ret) {
-    /* It can be canceled by the user, even without a cancellable. */
-    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-      g_warning ("Failed to open file via flatpak portal: %s", error->message);
-    g_error_free (error);
-  }
-
-  file = g_file_new_for_path (path);
-  g_free (path);
-
-  g_file_delete_async (file, G_PRIORITY_LOW, NULL, tmp_source_delete_cb, NULL);
-}
-
-static void
-save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointer data)
-{
-  const char *uri;
-  GFile *file;
-  GError *error = NULL;
-
-  g_output_stream_close_finish (ostream, result, &error);
-  if (error) {
-    g_warning ("Unable to close file: %s", error->message);
-    g_error_free (error);
-    delete_tmp_source_file (ostream);
-    return;
-  }
-
-  uri = (const char *)g_object_get_data (G_OBJECT (ostream), "ephy-save-temp-source-uri");
-
-  file = g_file_new_for_uri (uri);
-
-  if (ephy_is_running_inside_flatpak ()) {
-    const char *path;
-
-    path = g_file_get_path (file);
-    ephy_open_file_via_flatpak_portal (path, NULL, portal_opened_cb, g_strdup (path));
-    goto out;
-  }
-
-  if (!ephy_file_launch_handler (file, gtk_get_current_event_time ())) {
-    /* Fallback to view the source inside the browser */
-    EphyEmbed *embed;
-
-    uri = (const char *)g_object_get_data (G_OBJECT (ostream),
-                                           "ephy-original-source-uri");
-    embed = (EphyEmbed *)g_object_get_data (G_OBJECT (ostream),
-                                            "ephy-save-temp-source-embed");
-    view_source_embedded (uri, embed);
-  }
-
-out:
-  g_object_unref (ostream);
-  g_object_unref (file);
-}
-
-static void
-save_temp_source_write_cb (GOutputStream *ostream, GAsyncResult *result, GString *data)
-{
-  GError *error = NULL;
-  gssize written;
-
-  written = g_output_stream_write_finish (ostream, result, &error);
-  if (error) {
-    g_string_free (data, TRUE);
-    g_warning ("Unable to write to file: %s", error->message);
-    g_error_free (error);
-
-    g_output_stream_close_async (ostream, G_PRIORITY_DEFAULT, NULL,
-                                 (GAsyncReadyCallback)save_temp_source_close_cb,
-                                 NULL);
-
-    return;
-  }
-
-  if (written == (gint)data->len) {
-    g_string_free (data, TRUE);
-
-    g_output_stream_close_async (ostream, G_PRIORITY_DEFAULT, NULL,
-                                 (GAsyncReadyCallback)save_temp_source_close_cb,
-                                 NULL);
-
-    return;
-  }
-
-  data->len -= written;
-  data->str += written;
-
-  g_output_stream_write_async (ostream,
-                               data->str, data->len,
-                               G_PRIORITY_DEFAULT, NULL,
-                               (GAsyncReadyCallback)save_temp_source_write_cb,
-                               data);
-}
-
-static void
-get_main_resource_data_cb (WebKitWebResource *resource, GAsyncResult *result, GOutputStream *ostream)
-{
-  guchar *data;
-  gsize data_length;
-  GString *data_str;
-  GError *error = NULL;
-
-  data = webkit_web_resource_get_data_finish (resource, result, &data_length, &error);
-  if (error) {
-    g_warning ("Unable to get main resource data: %s", error->message);
-    g_error_free (error);
-    delete_tmp_source_file (ostream);
-    return;
-  }
-
-  /* We create a new GString here because we need to make sure
-   * we keep writing in case of partial writes */
-  data_str = g_string_new_len ((gchar *)data, data_length);
-  g_free (data);
-
-  g_output_stream_write_async (ostream,
-                               data_str->str, data_str->len,
-                               G_PRIORITY_DEFAULT, NULL,
-                               (GAsyncReadyCallback)save_temp_source_write_cb,
-                               data_str);
-}
-
-static void
-save_temp_source_replace_cb (GFile *file, GAsyncResult *result, EphyEmbed *embed)
-{
-  EphyWebView *view;
-  WebKitWebResource *resource;
-  GFileOutputStream *ostream;
-  GError *error = NULL;
-
-  ostream = g_file_replace_finish (file, result, &error);
-  if (error) {
-    g_warning ("Unable to replace file: %s", error->message);
-    g_error_free (error);
-    return;
-  }
-
-  g_object_set_data_full (G_OBJECT (ostream),
-                          "ephy-save-temp-source-uri",
-                          g_file_get_uri (file),
-                          g_free);
-
-  view = ephy_embed_get_web_view (embed);
-
-  g_object_set_data_full (G_OBJECT (ostream),
-                          "ephy-original-source-uri",
-                          g_strdup (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (view))),
-                          g_free),
-
-  g_object_set_data_full (G_OBJECT (ostream),
-                          "ephy-save-temp-source-embed",
-                          g_object_ref (embed),
-                          g_object_unref);
-
-  resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
-  webkit_web_resource_get_data (resource, NULL,
-                                (GAsyncReadyCallback)get_main_resource_data_cb,
-                                ostream);
-}
-
-static void
-save_temp_source (EphyEmbed *embed,
-                  guint32    user_time)
-{
-  GFile *file;
-  char *tmp, *base;
-
-  if (ephy_is_running_inside_flatpak ()) {
-    /* It has to go here because the portal has no access to our tmpfs.
-     * This means we have to delete it manually! */
-    base = g_build_filename (g_get_user_cache_dir (), "tmp", "viewsourceXXXXXX", NULL);
-  } else {
-    const char *static_temp_dir;
-
-    static_temp_dir = ephy_file_tmp_dir ();
-    if (static_temp_dir == NULL)
-      return;
-
-    base = g_build_filename (static_temp_dir, "viewsourceXXXXXX", NULL);
-  }
-
-  tmp = ephy_file_tmp_filename (base, "html");
-  g_free (base);
-
-  if (tmp == NULL)
-    return;
-
-  file = g_file_new_for_path (tmp);
-  g_file_replace_async (file, NULL, FALSE,
-                        G_FILE_CREATE_REPLACE_DESTINATION | G_FILE_CREATE_PRIVATE,
-                        G_PRIORITY_DEFAULT, NULL,
-                        (GAsyncReadyCallback)save_temp_source_replace_cb,
-                        embed);
-
-  g_object_unref (file);
-  g_free (tmp);
-}
-
-void
-window_cmd_page_source (GSimpleAction *action,
-                        GVariant      *parameter,
-                        gpointer       user_data)
-{
-  EphyWindow *window = user_data;
-  EphyEmbed *embed;
-  const char *address;
-  guint32 user_time;
-
-  embed = ephy_embed_container_get_active_child
-            (EPHY_EMBED_CONTAINER (window));
-  g_assert (embed != NULL);
-
-  address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
-
-  if (g_settings_get_boolean (EPHY_SETTINGS_MAIN,
-                              EPHY_PREFS_INTERNAL_VIEW_SOURCE)) {
-    view_source_embedded (address, embed);
-    return;
-  }
-
-  user_time = gtk_get_current_event_time ();
-
-  if (g_str_has_prefix (address, "file://")) {
-    GFile *file;
-
-    file = g_file_new_for_uri (address);
-    ephy_file_launch_handler (file, user_time);
-
-    g_object_unref (file);
-  } else {
-    save_temp_source (embed, user_time);
-  }
-}
-
 void
 window_cmd_toggle_inspector (GSimpleAction *action,
                              GVariant      *parameter,
@@ -2197,6 +2109,30 @@ window_cmd_go_home (GSimpleAction *action,
                   EPHY_LINK_HOME_PAGE);
 }
 
+void
+window_cmd_go_content (GSimpleAction *action,
+                       GVariant      *parameter,
+                       gpointer       user_data)
+{
+  GtkWidget *stack;
+  EphyWindow *window = EPHY_WINDOW (user_data);
+
+  stack = ephy_window_get_stack (window);
+  gtk_stack_set_visible_child_name (GTK_STACK (stack), "content");
+}
+
+void
+window_cmd_go_tabs_view (GSimpleAction *action,
+                         GVariant      *parameter,
+                         gpointer       user_data)
+{
+  GtkWidget *stack;
+  EphyWindow *window = EPHY_WINDOW (user_data);
+
+  stack = ephy_window_get_stack (window);
+  gtk_stack_set_visible_child_name (GTK_STACK (stack), "tabs");
+}
+
 void
 window_cmd_change_browse_with_caret_state (GSimpleAction *action,
                                            GVariant      *state,
@@ -2247,6 +2183,11 @@ window_cmd_change_fullscreen_state (GSimpleAction *action,
   gboolean active;
 
   active = g_variant_get_boolean (state);
+
+  /* This is performed only here because we don't want it occuring when a window
+   * enters fullscreen mode for some other reason other than action activation.
+   * E.g. we don't want it appearing for fullscreen video.
+   */
   ephy_window_show_fullscreen_header_bar (window);
 
   if (active)
@@ -2385,6 +2326,7 @@ window_cmd_tabs_close (GSimpleAction *action,
   EphyWindow *window = user_data;
   GtkWidget *notebook;
   EphyEmbed *embed;
+  gboolean pinned;
 
   notebook = ephy_window_get_notebook (window);
 
@@ -2397,7 +2339,10 @@ window_cmd_tabs_close (GSimpleAction *action,
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
 
-  g_signal_emit_by_name (notebook, "tab-close-request", embed);
+  pinned = ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed);
+
+  if (!pinned)
+    g_signal_emit_by_name (notebook, "tab-close-request", embed);
 }
 
 void
@@ -2416,7 +2361,9 @@ window_cmd_tabs_close_left (GSimpleAction *action,
 
   for (int i = 0; i < current_page_no; i++) {
     embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-    pages_to_close = g_slist_prepend (pages_to_close, embed);
+
+    if (!ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed))
+      pages_to_close = g_slist_prepend (pages_to_close, embed);
   }
 
   for (GSList *l = pages_to_close; l != NULL; l = l->next) {
@@ -2444,7 +2391,9 @@ window_cmd_tabs_close_right (GSimpleAction *action,
 
   for (int i = current_page_no + 1; i < n_pages; i++) {
     embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-    pages_to_close = g_slist_prepend (pages_to_close, embed);
+
+    if (!ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed))
+      pages_to_close = g_slist_prepend (pages_to_close, embed);
   }
 
   for (GSList *l = pages_to_close; l != NULL; l = l->next) {
@@ -2473,7 +2422,9 @@ window_cmd_tabs_close_others (GSimpleAction *action,
   for (int i = 0; i < n_pages; i++) {
     if (i != current_page) {
       embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-      pages_to_close = g_slist_prepend (pages_to_close, embed);
+
+      if (!ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed))
+        pages_to_close = g_slist_prepend (pages_to_close, embed);
     }
   }
 
@@ -2664,3 +2615,37 @@ window_cmd_new_tab_from_clipboard (GSimpleAction *action,
                               (GtkClipboardTextReceivedFunc)clipboard_text_received_cb,
                               g_object_ref (ephy_window));
 }
+
+void
+window_cmd_tabs_pin (GSimpleAction *action,
+                     GVariant      *parameter,
+                     gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyNotebook *notebook;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  notebook = EPHY_NOTEBOOK (ephy_window_get_notebook (window));
+
+  ephy_notebook_tab_set_pinned (notebook, GTK_WIDGET (embed), TRUE);
+}
+
+void
+window_cmd_tabs_unpin (GSimpleAction *action,
+                       GVariant      *parameter,
+                       gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyNotebook *notebook;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  notebook = EPHY_NOTEBOOK (ephy_window_get_notebook (window));
+
+  ephy_notebook_tab_set_pinned (notebook, GTK_WIDGET (embed), FALSE);
+}
