@@ -62,7 +62,9 @@ server_callback (SoupServer        *server,
 }
 
 static void
-load_changed_cb (WebKitWebView *view, WebKitLoadEvent load_event, GMainLoop *loop)
+load_changed_cb (WebKitWebView   *view,
+                 WebKitLoadEvent  load_event,
+                 GMainLoop       *loop)
 {
   char *expected_url;
   const char *loaded_url;
@@ -71,7 +73,7 @@ load_changed_cb (WebKitWebView *view, WebKitLoadEvent load_event, GMainLoop *loo
     return;
 
   expected_url = g_object_get_data (G_OBJECT (view), "test.expected_url");
-  g_assert (expected_url != NULL);
+  g_assert_nonnull (expected_url);
 
   loaded_url = webkit_web_view_get_uri (view);
   g_assert_cmpstr (loaded_url, ==, expected_url);
@@ -127,7 +129,6 @@ static const URLTest test_load_url[] = {
   /* { "about:epiphany", "ephy-about:epiphany" }, */
   /* { "about:applications", "ephy-about:applications" }, */
   /* { "about:memory", "ephy-about:memory" }, */
-  /* { "about:plugins", "ephy-about:plugins" }, */
 };
 
 /* Tests that EphyWebView is successfully loading the given URL. */
@@ -213,7 +214,7 @@ test_ephy_web_view_non_search_regex (void)
     g_test_message ("Regex failed: %s", error->message);
     g_error_free (error);
   }
-  g_assert (regex_non_search);
+  g_assert_nonnull (regex_non_search);
 
   regex_domain = g_regex_new (EPHY_WEB_VIEW_DOMAIN_REGEX,
                               0, G_REGEX_MATCH_NOTEMPTY, &error);
@@ -222,7 +223,7 @@ test_ephy_web_view_non_search_regex (void)
     g_test_message ("Regex failed: %s", error->message);
     g_error_free (error);
   }
-  g_assert (regex_domain);
+  g_assert_nonnull (regex_domain);
 
   for (i = 0; i < G_N_ELEMENTS (test_non_search_regex); i++) {
     RegexTest test;
@@ -233,8 +234,8 @@ test_ephy_web_view_non_search_regex (void)
                     test.match ? "NO SEARCH" : "SEARCH",
                     test.url);
 
-    g_assert (g_regex_match (regex_non_search, test.url, 0, NULL) == test.match ||
-              g_regex_match (regex_domain, test.url, 0, NULL) == test.match);
+    g_assert_true (g_regex_match (regex_non_search, test.url, 0, NULL) == test.match ||
+                   g_regex_match (regex_domain, test.url, 0, NULL) == test.match);
   }
 
   g_regex_unref (regex_non_search);
@@ -307,7 +308,7 @@ test_ephy_web_view_normalize_or_autosearch (void)
                                          "org.gnome.Epiphany.EphyWebViewTest",
                                          "http://duckduckgo.com/?q=%s&t=epiphany",
                                          "");
-  g_assert (ephy_search_engine_manager_set_default_engine (manager, "org.gnome.Epiphany.EphyWebViewTest"));
+  g_assert_true (ephy_search_engine_manager_set_default_engine (manager, "org.gnome.Epiphany.EphyWebViewTest"));
   verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_ddg, G_N_ELEMENTS (normalize_or_autosearch_test_ddg));
 
   ephy_search_engine_manager_modify_engine (manager,
@@ -315,7 +316,7 @@ test_ephy_web_view_normalize_or_autosearch (void)
                                             "http://www.google.com/?q=%s",
                                             "");
 
-  g_assert (ephy_search_engine_manager_set_default_engine (manager, default_engine));
+  g_assert_true (ephy_search_engine_manager_set_default_engine (manager, default_engine));
   verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_google, G_N_ELEMENTS (normalize_or_autosearch_test_google));
 
   ephy_search_engine_manager_delete_engine (manager, "org.gnome.Epiphany.EphyWebViewTest");
@@ -325,7 +326,9 @@ test_ephy_web_view_normalize_or_autosearch (void)
 }
 
 static void
-quit_main_loop_when_load_finished (WebKitWebView *view, WebKitLoadEvent load_event, GMainLoop *loop)
+quit_main_loop_when_load_finished (WebKitWebView   *view,
+                                   WebKitLoadEvent  load_event,
+                                   GMainLoop       *loop)
 {
   if (load_event != WEBKIT_LOAD_FINISHED)
     return;
@@ -395,8 +398,7 @@ test_ephy_web_view_provisional_load_failure_updates_back_forward_list (void)
 
   ensure_back_forward_list_changes (loop);
 
-  g_assert (webkit_back_forward_list_get_current_item (
-              webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view))));
+  g_assert_nonnull (webkit_back_forward_list_get_current_item (webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view))));
 
   g_assert_cmpstr (bad_url, ==, webkit_back_forward_list_item_get_uri (
                      webkit_back_forward_list_get_current_item (
@@ -429,7 +431,7 @@ test_ephy_web_view_error_pages_not_stored_in_history (void)
   bad_url = "http://localhost:2984375932/";
 
   history_service = ephy_embed_shell_get_global_history_service (embed_shell);
-  g_assert (history_service);
+  g_assert_nonnull (history_service);
   g_signal_connect (history_service, "visit-url",
                     G_CALLBACK (visit_url_cb), NULL);
 
@@ -446,7 +448,8 @@ test_ephy_web_view_error_pages_not_stored_in_history (void)
 }
 
 int
-main (int argc, char *argv[])
+main (int   argc,
+      char *argv[])
 {
   int ret;
   SoupServer *server;
@@ -456,7 +459,7 @@ main (int argc, char *argv[])
   ephy_debug_init ();
 
   if (!ephy_file_helpers_init (NULL,
-                               EPHY_FILE_HELPERS_PRIVATE_PROFILE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
+                               EPHY_FILE_HELPERS_TESTING_MODE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
                                NULL)) {
     g_debug ("Something wrong happened with ephy_file_helpers_init()");
     return -1;
